// =======================================================================
//  File: EntityConfig.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Resources;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;

using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

namespace Microsoft.Crm.Sandbox.Mobile
{
    /// <summary>
    /// This page allows the user to set various options on the entity as well as specify
    /// the attributes used in the Mobile Client.
    /// </summary>
    public partial class EntityConfig : System.Web.UI.Page
    {
        private ArrayList _attributeFields;
        private ArrayList _selectedFields;
     
        private XmlDocument _xmlDoc;
        private string _entityName;

        protected void Page_Load(object sender, EventArgs e)
        {            
            // Load draft XML and do not resolve namespaces
            _xmlDoc = new XmlDocument();
			_xmlDoc.XmlResolver = null;
            _xmlDoc.Load(ConfigurationManager.AppSettings["DraftXmlLocation"]);
           

            // If the page is on its first load retrieve the entity name from the query string
            if (ConfigEntityName.Value == null || ConfigEntityName.Value.Length == 0)
            {
                _entityName = Request.QueryString["sEntityName"];
                ConfigEntityName.Value = HttpUtility.HtmlEncode(_entityName); 
            }
            else // Retrieve the entity name from the form
            {
                _entityName = ConfigEntityName.Value;
            }

            // If page was called in some odd fashion stop page load
            if (_entityName == null || _entityName.Length == 0)
            {
				throw new ArgumentNullException("EntityName is a required parameter.");			
            }           

            // If save was called, save the configuration
            if (IsPostBack)
            {
                SaveEntityConfiguration();
				this.RefreshHomePage();
            }

            // If we are closing the window we do not want to run any of this logic
            if (!CloseWindow.Checked)
            {             
                // Get available and selected fields
                GetAttributeFields();              

                // Filter selected fields to be in correct order
                SortSelectedFields();

                // Render attribute list tables
                RenderAttributeFieldLists();

                // Disable editable buttons for certain entities
                bool setRadio = DisableRadioButtons();

                // Ignore value specified in config if the buttons are disabled
                if (setRadio)
                {
                    SetReadioButtons();
                }
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            // If we are saving and closing, do not render the page
            if (IsPostBack && CloseWindow.Checked)
            {
                Response.Clear();
                Response.Write("<script language=\"javascript\">window.close();</script>");
                this.RefreshHomePage();
                
                Response.End();
            }
            else
            {
                base.Render(writer);
            }
        }

        // Render out resources from resource file
        public void RenderResources()
        {
            Title = Utils.GetResource("DialogTitle");

            // Render out javascript localized variables
			Response.Write("<script language=\"javascript\">var CHANGES_WITHOUT_SAVE_WARNING =\"");
			Response.Write(Utils.GetResource("ChangesWithoutSaveWarning"));
			Response.Write("\"; var REMOVE_REQUIRED_FIELD_WARNING =\"");
			Response.Write(Utils.GetResource("RemoveRequiredFieldWarning"));
			Response.Write("\"; </script>");
			
            EntityConfigNameTitle.InnerText = Utils.GetResource("EntityConfigNameTitle") + " : " + Utils.GetDisplayName(_entityName);
            CanEditTitle.InnerText = Utils.GetResource("CanEditTitle");

            // Changing the innertext removes innerHTML as well, so we need to make sure that the image is added
			SaveButton.Attributes.Add("title", Utils.GetResource("SaveButtonToolTip"));
           
            SaveAndCloseButton.InnerHtml = string.Format("<div class=\"mnuBtn\"><img src=\"/_imgs/ico/16_saveClose.gif\" class=\"mnuBtn\">&nbsp;{0}</div>", Utils.GetResource("SaveAndCloseButtonText"));
			SaveAndCloseButton.Attributes.Add("title", Utils.GetResource("SaveAndCloseButtonToolTip"));
			
			CloseButton.Attributes.Add("title", Utils.GetResource("CloseButtonToolTip"));
			
            GeneralTab.InnerText = Utils.GetResource("GeneralTabText");

            EntityShouldBeText.InnerText = Utils.GetResource("EntityShouldBeText") + ":";
            ReadOnlyText.InnerText = Utils.GetResource("ReadOnlyText");
            ReadOnlyDescription.InnerText = Utils.GetResource("ReadOnlyDescription");

            EditableText.InnerText = Utils.GetResource("EditableText");
            EditableDescription.InnerText = Utils.GetResource("EditableDescription");

            SelectFieldsToShowTitle.InnerText = Utils.GetResource("SelectFieldsToShowTitle");
            SelectFieldsToShowDescription.InnerText = Utils.GetResource("SelectFieldsToShowDescription");

            AvailableAttributesText.InnerText = Utils.GetResource("AvailableAttributesText");
            SelectedAttributesText.InnerText = Utils.GetResource("SelectedAttributesText");

            AddButton.InnerText = Utils.GetResource("AddButtonText");
            RemoveButton.InnerText = Utils.GetResource("RemoveButtonText");

            MoveUpButton.InnerText = Utils.GetResource("MoveUpButtonText");
            MoveDownButton.InnerText = Utils.GetResource("MoveDownButtonText");

            EditLabelButton.InnerText = Utils.GetResource("EditLabelButtonText");

            CurrentLabelText.InnerText = Utils.GetResource("CurrentLabelText") + ":";
        }

        

        /// <summary>
        /// Save the current configuration to the draft XML
        /// </summary>
        private void SaveEntityConfiguration()
        {
            // Retrieve the current node from the draft XML
            XmlNode currentEntityNode = _xmlDoc.DocumentElement.SelectSingleNode(string.Format("Entities/Entity[@name='{0}']", _entityName));

            // Create new XML doc to hold the submitted XML and do not resolve namespaces
            XmlDocument formSubmitXml = new XmlDocument();
			formSubmitXml.XmlResolver = null;  
            formSubmitXml.LoadXml(FormSubmitXml.Value);                  

            // Change the readonly value to be what is on the form
            currentEntityNode.Attributes["readOnly"].Value = formSubmitXml.DocumentElement.Attributes["readOnly"].Value;

            // Delete the fields container and readd it empty
            XmlNode oldXmlFieldsNode = currentEntityNode.SelectSingleNode("Fields");
            currentEntityNode.RemoveChild(oldXmlFieldsNode);

            XmlNode newXmlFieldsNode = _xmlDoc.CreateElement("Fields");
            currentEntityNode.AppendChild(newXmlFieldsNode);

            XmlNodeList newXmlFields = formSubmitXml.DocumentElement.SelectNodes("Fields/Field");

            // Add all fields currently on the form 
            foreach (XmlNode newXmlField in newXmlFields)
            {
                XmlElement newXmlFieldElement = _xmlDoc.CreateElement("Field");
                newXmlFieldElement.SetAttribute("name", newXmlField.Attributes["name"].Value);
                newXmlFieldElement.SetAttribute("label", newXmlField.Attributes["label"].Value);
                
                newXmlFieldsNode.AppendChild(newXmlFieldElement);
            }

            // If the draft XML file exists, delete it and replace it with the new configuration
            if (File.Exists(ConfigurationManager.AppSettings["DraftXmlLocation"]))
            {
                File.Delete(ConfigurationManager.AppSettings["DraftXmlLocation"]);

                using (StreamWriter sw = File.AppendText(ConfigurationManager.AppSettings["DraftXmlLocation"]))
                {
                    sw.Write(_xmlDoc.InnerXml.ToString());
                }
            }
        }        

        /// <summary>
        /// Retrieve available and selected fields
        /// </summary>
        private void GetAttributeFields()
        {
			// Create MetadataService
			MetadataService metadataService = new MetadataService();
			metadataService.Credentials = System.Net.CredentialCache.DefaultCredentials;

            // Retrieve entity
            EntityMetadata entity = metadataService.RetrieveEntityMetadata(_entityName, EntityFlags.IncludeAttributes);

            // Intialize arraylists to be used for fields
            _attributeFields = new ArrayList();
            _selectedFields = new ArrayList();

            // Iterate through all the entity's attributes
            for (int i = 0; i < entity.Attributes.Length; i++)
            {
                // If the attribute is valid for form and it has a display name add it to the list
                if ((Utils.CheckBit((uint)entity.Attributes[i].DisplayMask, (uint)DisplayMasks.ValidForForm) && 
                    entity.Attributes[i].DisplayName != null && 
                    entity.Attributes[i].DisplayName.Length != 0) ||
                    (_entityName == EntityName.annotation.ToString() &&
                    entity.Attributes[i].DisplayName != null &&                    
                    entity.Attributes[i].DisplayName.Length != 0))
                {
                    Field field = new Field();

                    field.SchemaName = entity.Attributes[i].Name;
                    field.DisplayName = entity.Attributes[i].DisplayName;
                    field.LabelName = entity.Attributes[i].DisplayName; // This is defaulted to the display name at first
                    field.RequiredLevel = entity.Attributes[i].RequiredLevel;                  

                    // If the field has already been selected, add it to the selected fields array instead
                    if (IsAttributeSelected(field.SchemaName))
                    {
                        _selectedFields.Add(field);
                    }
                    else
                    {
                        _attributeFields.Add(field);
                    }
                }
            }

            // Sort the available fields array alphabetically
            _attributeFields.Sort(new FieldComparer());    
        }

        /// <summary>
        /// Determines if the attribute has already been selected
        /// </summary>
        /// <param name="schemaName">Schema name of the attribute</param>
        /// <returns>If the attribute was found in the draft XML</returns>
        private bool IsAttributeSelected(string attributeName)
        {
            // Retrieve attribute from draft XML
            XmlNode selectedAttribute = _xmlDoc.DocumentElement.SelectSingleNode(string.Format("Entities/Entity[@name='{0}']/Fields/Field[@name='{1}']", _entityName, attributeName));

            // If the attribute was found return true, otherwise false
            if (selectedAttribute != null)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Filter selected fields as they appear in the draft XML
        /// </summary>
        private void SortSelectedFields()
        {
            // Retrieve entity from draft XML
            XmlNode entity = _xmlDoc.DocumentElement.SelectSingleNode(string.Format("Entities/Entity[@name='{0}']", _entityName));

            XmlNodeList xmlFields = entity.SelectNodes("Fields/Field");

            ArrayList tempSelectedFields = new ArrayList();

            // For each XML field in the config add it to a temp array in the proper order
            // NOTE: We are doing it this way is we need to hit the metadata to retrieve the
            // current display name and required level and cannot just add fields from the config
            foreach (XmlNode xmlField in xmlFields)
            {
                Field field = new Field();
                
                // Create dummy field
                field.SchemaName = string.Empty;
                field.DisplayName = string.Empty;
                field.LabelName = string.Empty;
                field.RequiredLevel = AttributeRequiredLevel.None;

                for (int i = 0; i < _selectedFields.Count; i++)
                {
                    field = (Field)_selectedFields[i];

                    // If the schema name matches, overwrite the label
                    if (field.SchemaName == xmlField.Attributes["name"].Value)
                    {
                        field.LabelName = xmlField.Attributes["label"].Value;
                        break;
                    }
                }

                // Add correct field
                tempSelectedFields.Add(field);
            }

            // Set selected fields to be the selected fields in the correct order with the correct labels
            _selectedFields = tempSelectedFields;
            
			// TODO: Look into encoding
            // Sets CurrentXML and FormSubmitXML to the current entity XML
            CurrentXml.Value = entity.OuterXml;
            FormSubmitXml.Value = entity.OuterXml;
        }

        /// <summary>
        /// Builds up HTML to be used as list table
        /// </summary>
        /// <param name="fieldList">Arraylist being used to render table</param>
        /// <returns>HTML of table</returns>
        private string BuildFieldTable(ArrayList fieldList)
        {
            StringBuilder htmlBuilder = new StringBuilder();

            htmlBuilder.Append("<table cellSpacing=\"0\" cellPadding=\"2\" width=\"100%\">");

            // Add every field in arraylist as a new row
            for (int i = 0; i < fieldList.Count; i++)
            {
                Field field = (Field)fieldList[i];

                htmlBuilder.AppendFormat("<tr name=\"{0}\" label=\"{1}\" req=\"{2}\" selected=\"0\" onclick=\"selectRow(true);\" ondblclick=\"MoveToOtherList();\" onkeydown=\"HandleKeyDown()\">", field.SchemaName, field.LabelName, (int)field.RequiredLevel);
                htmlBuilder.AppendFormat("<td class=\"sel\" noWrap>{0}</td></tr>", field.DisplayName);
            }

            htmlBuilder.Append("</table>");

            return htmlBuilder.ToString();
        }

        /// <summary>
        /// Renders attribute list tables
        /// </summary> 
        private void RenderAttributeFieldLists()
        {
            // Set available fields table
            objList.InnerHtml = BuildFieldTable(_attributeFields);

            // Set selected fields table
            rtnObjList.InnerHtml = BuildFieldTable(_selectedFields);
        }       

		// TODO: Look into enabling instead of disabling
        /// <summary>
        /// Disable the editable buttons on readonly entities
        /// </summary>
        /// <returns>Determines if the editable buttons should be disabled</returns>
        private bool DisableRadioButtons()
        {
            // If the entity is readonly specified, do not allow the user to change it
            if (_entityName == EntityName.annotation.ToString() ||
                _entityName == EntityName.appointment.ToString() ||
                _entityName == EntityName.email.ToString() ||
                _entityName == EntityName.invoice.ToString() ||
                _entityName == EntityName.invoicedetail.ToString() ||
                _entityName == EntityName.salesorder.ToString() ||
                _entityName == EntityName.salesorderdetail.ToString() ||
                _entityName == EntityName.quote.ToString() ||
                _entityName == EntityName.quotedetail.ToString() ||
                _entityName == EntityName.systemuser.ToString() ||
                _entityName == EntityName.list.ToString() ||
                _entityName == EntityName.campaign.ToString())
            {
                radReadOnly.Disabled = true;
                radEditable.Disabled = true;

                return false;
            }

            return true;
        }

        /// <summary>
        /// Set the value of the editable buttons
        /// </summary>
        private void SetReadioButtons()
        {
            XmlNode entity = _xmlDoc.DocumentElement.SelectSingleNode(string.Format("Entities/Entity[@name='{0}']", _entityName));

            // If the entity is editable, set the radio buttons (default is readonly)
            if (entity.Attributes["readOnly"].Value == "0")
            {
                radReadOnly.Checked = false;
                radEditable.Checked = true;
            }
        }
        
        private void RefreshHomePage()
        {
			//Assign the url of the home page window that opened this page
			//to the homepage. Assigned is used instead of reload incase the
			//publish or any other actions have placed on the homepage previously.
			//This prevents the reposting of form data on the homepage. 
			Response.Write("<script language=\"javascript\">window.opener.location.assign(\"HomePage.aspx\");</script>");
        }
    }
}
