// =======================================================================
//  File: HomePage.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Collections;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;

using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

namespace Microsoft.Crm.Sandbox.Mobile
{
    /// <summary>
    /// This is the home page that is used for adding, removing, and reordering entities for the Mobile Client
    /// </summary>
    public partial class HomePage : System.Web.UI.Page
    {
        private const int CUSTOM_OBJECT_TYPE_CODE = 10000;

        private XmlDocument _xmlDoc;        

        protected void Page_Load(object sender, EventArgs e)
        {
            // Check if XML files exist
            CheckXmlFiles();

            // Load current draft XML
            _xmlDoc = new XmlDocument();
            _xmlDoc.XmlResolver = null;
            _xmlDoc.Load(ConfigurationManager.AppSettings["DraftXmlLocation"]);

            // If save was called, save the configuration
            // Note: Publishing does a postback as well, so we check if we are submitting XML as well
            if (IsPostBack && FormSubmitXml.Value != null && FormSubmitXml.Value.Length != 0)
            {
                SaveHomePageConfiguration();
            }

            // Checks to see if published and draft files are different
            CompareXmlFiles();

            // Render table (this is last so it load the current configuration after the save)
            RenderEntityList();
        }

        /// <summary>
        /// Creates XML Files if they do not exist
        /// </summary>
        private void CheckXmlFiles()
        {
            // Create a published XML File if one doesn't already exist
            if (!File.Exists(ConfigurationManager.AppSettings["PublishedXmlLocation"]))
            {
                XmlDocument newPublishedXmlDoc = new XmlDocument();                 

                XmlElement mobileClientElement = newPublishedXmlDoc.CreateElement("MobileClient");
                newPublishedXmlDoc.AppendChild(mobileClientElement);

                XmlElement entitiesElement = newPublishedXmlDoc.CreateElement("Entities");
                mobileClientElement.AppendChild(entitiesElement);

                // Create a dummy published XML file with no entities added to the configuration
                using (StreamWriter sw = File.AppendText(ConfigurationManager.AppSettings["PublishedXmlLocation"]))
                {
                    sw.Write(newPublishedXmlDoc.InnerXml.ToString());
                }
            }

            // Create the draft XML from the published version if the file doesn't already exist
            if (!File.Exists(ConfigurationManager.AppSettings["DraftXmlLocation"]))
            {
                File.Copy(ConfigurationManager.AppSettings["PublishedXmlLocation"], ConfigurationManager.AppSettings["DraftXmlLocation"]);
            }
        }

        /// <summary>       
        /// Save the current configuration to the draft XML - 
		/// 3 XML files are used to shuffle around the needed nodes. One is the form submit XML that
		/// lays out the order of the home page and if entities have been added. One is the current Draft
		/// XML that is being used, and the last one is going to be the merged result of the first two.
        /// </summary>
        private void SaveHomePageConfiguration()
        {
            // Create new XML doc to hold the submitted XML and do not resolve namespaces
            XmlDocument formSubmitXml = new XmlDocument();
            formSubmitXml.XmlResolver = null;
            formSubmitXml.LoadXml(FormSubmitXml.Value);            

            // Retrieve the existing and added entities
            XmlNodeList existingEntities = formSubmitXml.DocumentElement.SelectNodes("ExistingEntities/Entity");
            XmlNodeList addedEntities = formSubmitXml.DocumentElement.SelectNodes("AddedEntities/Entity");

            // Create temporary XML doc that will combine the fields of the current draft XML but the 
            // order of the submitted version plus the added entities
            XmlDocument tempXmlDoc = new XmlDocument();

            // Create document element and entities container
            XmlElement tempMobileElement = tempXmlDoc.CreateElement("MobileClient");
            XmlElement tempEntitiesElement = tempXmlDoc.CreateElement("Entities");

            tempXmlDoc.AppendChild(tempMobileElement);
            tempMobileElement.AppendChild(tempEntitiesElement);

            // For every entity already added to the configuration (in new order)
            foreach (XmlNode existingEntity in existingEntities)
            {
                // Get full entity node from current XML which will include all attributes
                XmlNode fullEntityNode = _xmlDoc.DocumentElement.SelectSingleNode(string.Format("Entities/Entity[@name='{0}']", existingEntity.Attributes["name"].Value));

                XmlElement newEntityElement = tempXmlDoc.CreateElement("Entity");
                newEntityElement.SetAttribute("name", fullEntityNode.Attributes["name"].Value);
                newEntityElement.SetAttribute("readOnly", fullEntityNode.Attributes["readOnly"].Value);

                XmlElement newEntityFields = tempXmlDoc.CreateElement("Fields");

                XmlNodeList fullEntityFields = fullEntityNode.SelectNodes("Fields/Field");

                // Add all the current fields to the entity
                foreach (XmlNode fullEntityField in fullEntityFields)
                {
                    XmlElement newEntityField = tempXmlDoc.CreateElement("Field");
                    newEntityField.SetAttribute("name", fullEntityField.Attributes["name"].Value);
                    newEntityField.SetAttribute("label", fullEntityField.Attributes["label"].Value);

                    newEntityFields.AppendChild(newEntityField);
                }

                // Add fields
                newEntityElement.AppendChild(newEntityFields);

                // Add entity
                tempEntitiesElement.AppendChild(newEntityElement);
            }

            // Add entities added from the Add Entity dialog to the configuration
            foreach (XmlNode addedEntity in addedEntities)
            {
                XmlElement addedEntityElement = tempXmlDoc.CreateElement("Entity");

                addedEntityElement.SetAttribute("name", addedEntity.Attributes["name"].Value);

                // Make the entity readonly by default
                addedEntityElement.SetAttribute("readOnly", "1");

                XmlElement fieldsElement = tempXmlDoc.CreateElement("Fields");

                // Create MetadataService
                MetadataService metadataService = new MetadataService();
                metadataService.Credentials = System.Net.CredentialCache.DefaultCredentials;

                // Retrieve entity
                EntityMetadata entity = metadataService.RetrieveEntityMetadata(addedEntity.Attributes["name"].Value, EntityFlags.IncludeAttributes);

                ArrayList attributeFields = new ArrayList();

                // Add only attributes that are required by the system, business, or recommended
                for (int i = 0; i < entity.Attributes.Length; i++)
                {
                    // Note do not have any attributes that are valid for the Form but we still want to be able to 
                    // configure them so we handle that special case here
                    // NOTE: In order for custom attributes to be added, they must be published
                    if ((Utils.CheckBit((uint)entity.Attributes[i].DisplayMask, (uint)DisplayMasks.ValidForForm) &&
                    entity.Attributes[i].DisplayName != null &&
                    entity.Attributes[i].DisplayName.Length != 0) ||
                    (entity.Name == EntityName.annotation.ToString() &&
                    entity.Attributes[i].DisplayName != null &&
                    entity.Attributes[i].DisplayName.Length != 0))
                    {
                        if (entity.Attributes[i].RequiredLevel == AttributeRequiredLevel.SystemRequired ||
                            entity.Attributes[i].RequiredLevel == AttributeRequiredLevel.Required ||
                            entity.Attributes[i].RequiredLevel == AttributeRequiredLevel.Recommended)
                        {
                            Field field = new Field();

                            field.SchemaName = entity.Attributes[i].Name;
                            field.DisplayName = entity.Attributes[i].DisplayName;
                            field.LabelName = entity.Attributes[i].DisplayName;
                            field.RequiredLevel = entity.Attributes[i].RequiredLevel;

                            attributeFields.Add(field);
                        }
                    }
                }
                       
                // Sort fields to be in alphabetical order
                attributeFields.Sort(new FieldComparer());

                // Add required attributes to entity
                for (int i = 0; i < attributeFields.Count; i++)
                {
                    XmlElement requiredFieldElement = tempXmlDoc.CreateElement("Field");

                    requiredFieldElement.SetAttribute("name", ((Field)attributeFields[i]).SchemaName);
                    requiredFieldElement.SetAttribute("label", ((Field)attributeFields[i]).LabelName);

                    fieldsElement.AppendChild(requiredFieldElement);
                }

                // Add fields
                addedEntityElement.AppendChild(fieldsElement);

                // Add entity
                tempEntitiesElement.AppendChild(addedEntityElement);
            }

            // Set temp XML to be current XML
            _xmlDoc = tempXmlDoc;

            // If there is already a draft XML file, delete it
            if (File.Exists(ConfigurationManager.AppSettings["DraftXmlLocation"]))
            {
                File.Delete(ConfigurationManager.AppSettings["DraftXmlLocation"]);
            }

            // Create a new draft XML file with the current configuration
            using (StreamWriter sw = File.AppendText(ConfigurationManager.AppSettings["DraftXmlLocation"]))
            {
                sw.Write(_xmlDoc.InnerXml.ToString());
            }
            
        }

        // Render out resources from resource file
        protected void RenderResources()
        {
            Title = Utils.GetResource("DialogTitle");

            // Render out javascript localized variables
            // TODO: Look into encoding
            Response.Write("<script language=\"javascript\">var MUST_SELECT_ENTITY_WARNING = \"");
            Response.Write(Utils.GetResource("MustSelectEntityWarning"));
            Response.Write("\"; var REMOVE_ENTITY_WARNING = \"");
            Response.Write(Utils.GetResource("RemoveEntityWarning"));
            Response.Write("\";</script>");

            HomePageTitle.InnerText = Utils.GetResource("HomePageTitle");
            HomePageDescription.InnerText = Utils.GetResource("HomePageDescription");

            AddButton.InnerText = Utils.GetResource("AddButtonText");
            RemoveButton.InnerText = Utils.GetResource("RemoveButtonText");

            MoveUpButton.InnerText = Utils.GetResource("MoveUpButtonText");
            MoveDownButton.InnerText = Utils.GetResource("MoveDownButtonText");

			EditButton.InnerText = Utils.GetResource("EditButtonText");

            PublishButton.InnerText = Utils.GetResource("PublishButtonText");

            LocationText.InnerText = Utils.GetResource("LocationText") + ":";
            MobileConfigurationLocationText.InnerText = Utils.GetResource("MobileConfigurationLocationText");
            AdvancedViewText.InnerText = Utils.GetResource("AdvancedViewText");
        }          

        /// <summary>
        /// Compare the draft and published XML files
        /// </summary>
        private void CompareXmlFiles()
        {
            XmlDocument publishedXml = new XmlDocument();
            publishedXml.XmlResolver = null;
            publishedXml.Load(ConfigurationManager.AppSettings["PublishedXmlLocation"]);

            // If both files are the same disable publishing
            if (publishedXml.OuterXml == _xmlDoc.OuterXml)
            {
                PublishButton.Disabled = true;               
            }
            else
            {
				PublishButton.Disabled = false;
            }
        }
        
        /// <summary>
        /// Renders out the entity list table
        /// </summary>
        private void RenderEntityList()
        {
            StringBuilder htmlBuilder = new StringBuilder();
            htmlBuilder.Append("<table cellspacing=\"0\" cellpadding=\"4\" style=\"table-layout: fixed\"><col width=\"24\" /><col />");

            XmlNodeList entities = _xmlDoc.DocumentElement.SelectNodes("Entities/Entity");

            // Add every entity that is found in the draft XML 
            foreach (XmlNode entity in entities)
            {
                string entityName = (entity.Attributes["name"].Value);
                int objectTypeCode = Utils.GetObjectTypeCode(entityName);

                htmlBuilder.AppendFormat("<tr class=\"grid\" name=\"{0}\" onclick=\"selectRow();\" ondblclick=\"EditEntity();\" onkeydown=\"HandleKeyDown();\">", entityName);
                htmlBuilder.Append("<td class=\"grid sel\" align=\"center\"><img src=\"");

                // If the entity is a system entity get the icon from the imgs folder, otherwise go to icon.aspx
                if (objectTypeCode < CUSTOM_OBJECT_TYPE_CODE) 
                {
                    htmlBuilder.AppendFormat("/_imgs/ico_16_{0}.gif", objectTypeCode.ToString(CultureInfo.InvariantCulture));
                }
                else
                {
                    htmlBuilder.AppendFormat("/_imgs/icon.aspx?objectTypeCode={0}&iconType=DBGridIcon&inProduction=1&cache=1", objectTypeCode.ToString(CultureInfo.InvariantCulture));
                }

                htmlBuilder.AppendFormat("\"></td><td class=\"grid sel\"><nobr>{0}</nobr></td></tr>", Utils.GetDisplayName(entityName));
            }

            htmlBuilder.Append("</table>");
                        
            rtnObjList.InnerHtml = htmlBuilder.ToString();
        }

        /// <summary>
        /// Publish the customizations when the user clicks the Publish Button
        /// </summary>
        protected void PublishCustomizations(object sender, EventArgs e)
        {
			// If there is already a published XML file, delete it
			if (File.Exists(ConfigurationManager.AppSettings["PublishedXmlLocation"]))
			{
				File.Delete(ConfigurationManager.AppSettings["PublishedXmlLocation"]);
			}

			// Copy the draft XML over to be the published XML
			File.Copy(ConfigurationManager.AppSettings["DraftXmlLocation"], ConfigurationManager.AppSettings["PublishedXmlLocation"]);
                    
			PublishButton.Disabled = true;
			
        }
        
    }
}
