// =======================================================================
//  File: EntityForm.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Drawing;
using System.Threading;
using System.Xml;
using System.Collections;
using System.Text.RegularExpressions;
using System.Globalization;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.Helpers;
using Microsoft.Crm.Sandbox.Mobile.Helpers.Collection;

/// <summary>
/// This class represents the form of an instance. The form
/// can be either readonly or in edit mode. The controls on this
/// form change depending on the data type in crm. In edit mode there is
/// an option to save.
/// </summary>
public class EntityForm : HtmlForm
{
    private bool _readOnly;
    private const string ENTITY_REQUEST_INDEX = "entity";
    private const string LOOKUP_PICKLIST_SUFFIX = "_Picklist";
    private const string LOOKUP_ENTITIES_SUFFIX = "_Entities";
    private const string LOOKUP_HIDDEN_SUFFIX = "_Lookup";
    private const string STATE_COLUMN = "statecode";
	private const int STRING_LIMIT = 100;
	private const int TEXTAREA_ROWS_LIMIT = 3;
    private EntityMetadata _eMeta; 
    private Mobile _mPage;
    private string _recordName;    
    private Guid _recordId;
	private CultureInfo _currentCulture;
	private string _masterPagePrefix;
	
    /// <summary>
    /// Metadata describing the current entity type
    /// </summary>
    public EntityMetadata CurrentEntityMetadata
    {
        get
        {
            return _eMeta;
        }
        set
        {
            _eMeta = value;
        }
    }

    /// <summary>
    /// Name of the current record (PrimaryField)
    /// </summary>
    public string RecordName
    {
        get
        {
            return _recordName;
        }
        set
        {
            _recordName = value;
        }
    }

    /// <summary>
    /// Guid value of the current record's PrimaryKey
    /// </summary>
    public Guid RecordId
    {
        get
        {
            return _recordId;
        }
        set
        {
            _recordId = value;
        }
    }
    
    /// <summary>
    /// Current Culture set by the culture of the organization
    /// </summary>
    public CultureInfo CurrentCulture
    {
		get
		{
			return _currentCulture;
		}
		set
		{
			_currentCulture = value;
		}	
    }

    public EntityForm(Mobile mPage, bool readOnly)
    {
        //Initialize member variables
        _readOnly = readOnly;
        _mPage = mPage;

        //Initialize the entity metadata of the entity instance we are in
		_eMeta = MetadataCache.GetEntityMetadata(_mPage.Request.QueryString[ENTITY_REQUEST_INDEX]);

        // Set Metadata to be used on page for BreadCrumbs
        this.CurrentEntityMetadata = _eMeta;

		// Get current user and their org culture, set on page
		User user = UserCache.GetCurrentUser();
		Organization org = OrganizationCache.GetOrganization(user);
		
		this.CurrentCulture = org.OrganizationCulture;
    }

	/// <summary>
	/// Method called on initialization of this form class
	/// </summary>
	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);

		//Initialize the table that the controls will be added
		HtmlTable formTable = new HtmlTable();

		//Get the current instance as a dynamic entity
		DynamicEntity de = this.GetCurrentEntity();

		//Get the list of field nodes from the config xml
		XmlNodeList fieldNodes = this.GetFieldNodes();

		// Master pages use prefixes for all of their controls and in order to properly retrieve
		// them using Request.Form we need this prefix added to the id we supply
		_masterPagePrefix = _mPage.Master.UniqueID + Mobile.MASTER_PAGE_CONTENT;

		//Loop through the field nodes and add the controls to the form table
		foreach (XmlNode fieldNode in fieldNodes)
		{
			XmlAttribute nameAttribute = (XmlAttribute)fieldNode.Attributes.GetNamedItem("name");
			XmlAttribute labelAttribute = (XmlAttribute)fieldNode.Attributes.GetNamedItem("label");

			if (nameAttribute != null)
			{
				AttributeMetadata attrib = MetadataCache.GetAttributeMetadata(_eMeta.Name, nameAttribute.Value);

				if (attrib != null)
				{
					//Get the property of the attribute
					Property property = MobileUtility.GetDynamicEntityProperty(de, attrib.Name);

					//Add the control to the form table
					AddAttributeControl(formTable, attrib, property, labelAttribute.Value);
				}
			}
		}

		// If the current record name is null, grab the "name" of the record
		if (this.RecordName == null)
		{
			StringProperty primaryField = (StringProperty)MobileUtility.GetDynamicEntityProperty(de, _eMeta.PrimaryField);

			if (primaryField != null)
			{
				this.RecordName = primaryField.Value;
			}
		}

		// If the current record id is empty, grab the Guid value of the key
		if (this.RecordId == Guid.Empty)
		{
			KeyProperty primaryKey = (KeyProperty)MobileUtility.GetDynamicEntityProperty(de, _eMeta.PrimaryKey);

			if (primaryKey != null)
			{
				this.RecordId = primaryKey.Value.Value;
			}
		}

		//if readonly, create and add the edit button for the instance 
		if (_readOnly)
		{
			if (!MobileUtility.IsEntityReadOnly(this.CurrentEntityMetadata.Name))
			{
				if (!MobileUtility.IsRecordActive(this.CurrentEntityMetadata.Name, MobileUtility.GetDynamicEntityPropertyValue(de, STATE_COLUMN), this.CurrentEntityMetadata.IsCustomEntity))
				{
					BoldLabel inactiveRecordLabel = new BoldLabel();
					inactiveRecordLabel.ForeColor = Color.Gray;
					inactiveRecordLabel.Text = MobileUtility.GetResource("RecordIsInactiveText");

					this.Controls.Add(inactiveRecordLabel);
				}
				else if (this.CurrentEntityMetadata.Name.Equals(EntityName.customeraddress.ToString()))
				{
					if (MobileUtility.DoesUserHavePrivilege(this.CurrentEntityMetadata.Name, PrivilegeType.Write))
					{
						this.AddEditLinkButton();
					}
				}
				else if (MobileUtility.DoesUserHaveRightsToRecord(this.CurrentEntityMetadata.Name, this.RecordId, AccessRights.WriteAccess) && MobileUtility.DoesUserHavePrivilege(this.CurrentEntityMetadata.Name, PrivilegeType.Write))
				{
					this.AddEditLinkButton();
				}
			}
		}

		//Add the table of controls to this form
		this.Controls.Add(formTable);

		//If not readonly, create and add a save button
		if (!_readOnly)
		{
			// Add HR before Save Button
			FormBar formBar = new FormBar();
			this.Controls.Add(formBar);

			Button saveButton = new Button();
			saveButton.Text = MobileUtility.GetResource("SaveButtonText");
			saveButton.Click += new System.EventHandler(this.Save_Click);

			//Add the save button to this form                
			this.Controls.Add(saveButton);
		}
	}

    /// <summary>
    /// Method to add the controls to the form table
    /// </summary>
    private void AddAttributeControl(HtmlTable formTable, AttributeMetadata attrib, Property property, string labelName)
    {
        //Initialize the rows and cells for the attribute
        HtmlTableRow labelRow = new HtmlTableRow();
        HtmlTableRow controlRow = new HtmlTableRow();
        HtmlTableRow validatorRow = new HtmlTableRow();
        HtmlTableCell validatorCell = new HtmlTableCell();
        HtmlTableCell labelCell = new HtmlTableCell();
        HtmlTableCell controlCell = new HtmlTableCell();


        //Set the label for the attribute				
        BoldLabel attribLabel = new BoldLabel();
        attribLabel.Text = labelName + ":";
        labelCell.Controls.Add(attribLabel);

        //Add the label control row to the table
        labelRow.Controls.Add(labelCell);
        formTable.Controls.Add(labelRow);

        //Evaluate each attribute type and add the appropriate controls and validators
        //depending on the read mode (readonly or edit)
        switch (attrib.Type)
        {
            case AttributeType.String:      StringProperty sProperty = (StringProperty)property;

                                            if (!this._readOnly)
                                            {
                                                //Initialize the string metadata based on this attribute
                                                StringAttributeMetadata sAttrib = (StringAttributeMetadata)attrib;

                                                //Check to see the maxlength of the string metadata for 
                                                //this attribute. The ntext fields in crm come back as 
                                                //StringProperty, so there is no way to differentiate between
                                                //a regular text and a ntext. So a textarea is created instead
                                                //of a textbox when the length of the metadata is over the
                                                //predefined amount
                                                if (sAttrib.MaxLength > STRING_LIMIT)
                                                {
                                                    HtmlTextArea textArea = new HtmlTextArea();
                                                    textArea.ID = attrib.Name;
                                                    textArea.Rows = TEXTAREA_ROWS_LIMIT;

                                                    if (sProperty != null)
                                                    {
                                                        textArea.Value = sProperty.Value;
                                                    }

                                                    controlCell.Controls.Add(textArea);
                                                }
                                                else
                                                {
                                                    //Initialize and set the properties of the control
                                                    TextBox textControl = new TextBox();
                                                    textControl.ID = attrib.Name;

                                                    if (sProperty != null)
                                                    {
                                                        textControl.Text = sProperty.Value;
                                                    }

                                                    //Set the control limits based on the crm data type attribute metadata
                                                    textControl.MaxLength = sAttrib.MaxLength;

                                                    //Add the textbox control
                                                    controlCell.Controls.Add(textControl);
                                                }
                                            }
                                            else
                                            {
                                                if (sProperty != null)
                                                {
                                                    //Add the read only label or link
                                                    this.AddReadOnlyControl(controlCell, sProperty.Value);
                                                }
                                            }

                                            break;

            case AttributeType.Lookup:      this.AddLookupControl(controlCell, validatorCell, property, attrib, formTable);
                                            break;

            case AttributeType.Customer:    this.AddLookupControl(controlCell, validatorCell, property, attrib, formTable);
                                            break;

            case AttributeType.Integer:     CrmNumberProperty nProperty = (CrmNumberProperty)property;

                                            if (!this._readOnly)
                                            {
                                                //Inititialize and set the properties of the control
                                                TextBox iControl = new TextBox();
                                                iControl.ID = attrib.Name;

                                                if (nProperty != null)
                                                {
                                                    iControl.Text = nProperty.Value.formattedvalue;
                                                }

                                                //Add the text box
                                                controlCell.Controls.Add(iControl);

                                                //Initialize the integer metadata based on this attribute
                                                IntegerAttributeMetadata iMeta = (IntegerAttributeMetadata)attrib;

					                            // Validator used to verify the integer value inputted is in the correct format and range     
                                                CustomValidator intCustomValidator = new CustomValidator();
                                                intCustomValidator.ControlToValidate = attrib.Name;
                                                intCustomValidator.EnableClientScript = false;
                                                intCustomValidator.ServerValidate += new ServerValidateEventHandler(this.ValidateNumber);
                                                intCustomValidator.Display = ValidatorDisplay.Dynamic;
                                                intCustomValidator.ErrorMessage = String.Format(MobileUtility.GetResource("NumericValidatorText"), iMeta.MinValue.ToString(("N" + 0), this.CurrentCulture), iMeta.MaxValue.ToString(("N" + 0), this.CurrentCulture));

					                            // Add the validator
                                                validatorCell.Controls.Add(intCustomValidator);                    
                                            }
                                            else
                                            {
                                                if (nProperty != null)
                                                {
                                                    //Add the readonly label
                                                    this.AddReadOnlyControl(controlCell, nProperty.Value.formattedvalue);
                                                }
                                            }
                                            break;

            case AttributeType.Float:       CrmFloatProperty fProperty = (CrmFloatProperty)property;

                                            if (!this._readOnly)
                                            {
                                                //Instantiate and set the properties of the control
                                                TextBox fControl = new TextBox();
                                                fControl.ID = attrib.Name;

                                                if (fProperty != null)
                                                {
                                                    fControl.Text = fProperty.Value.formattedvalue;
                                                }

                                                //Add the textbox	
                                                controlCell.Controls.Add(fControl);

                                                //Initialize the float metadata based on this attribute
                                                FloatAttributeMetadata fMeta = (FloatAttributeMetadata)attrib;

					                            // Validator used to verify the float value inputted is in the correct format and range     
					                            CustomValidator floatCustomValidator = new CustomValidator();
					                            floatCustomValidator.ControlToValidate = attrib.Name;
					                            floatCustomValidator.EnableClientScript = false;
					                            floatCustomValidator.ServerValidate += new ServerValidateEventHandler(this.ValidateNumber);
					                            floatCustomValidator.Display = ValidatorDisplay.Dynamic;
					                            floatCustomValidator.ErrorMessage = String.Format(MobileUtility.GetResource("NumericValidatorText"), fMeta.MinValue.ToString(("N" + fMeta.Precision), this.CurrentCulture), fMeta.MaxValue.ToString(("N" + fMeta.Precision), this.CurrentCulture));

					                            // Add the validator
					                            validatorCell.Controls.Add(floatCustomValidator);                    
                                            }
                                            else
                                            {
                                                if (fProperty != null)
                                                {
                                                    //Add the read only label
                                                    this.AddReadOnlyControl(controlCell, fProperty.Value.formattedvalue);
                                                }
                                            }
                                            break;

            case AttributeType.DateTime:    CrmDateTimeProperty dProperty = (CrmDateTimeProperty)property;

                                            if (!this._readOnly)
                                            {
                                                //Initialize and set the properties of the control
                                                TextBox dControl = new TextBox();
                                                dControl.ID = attrib.Name;

                                                if (dProperty != null)
                                                {
                                                    dControl.Text = dProperty.Value.date;
                                                }                     

                                                //Add the control
                                                controlCell.Controls.Add(dControl);                    
                                                                    
                                                // Validator used to verify the date inputted is valid                    
                                                CustomValidator dateValidator = new CustomValidator();
                                                dateValidator.ControlToValidate = attrib.Name;
                                                dateValidator.EnableClientScript = false;
                                                dateValidator.ServerValidate += new ServerValidateEventHandler(this.ValidateDate);
                                                dateValidator.Display = ValidatorDisplay.Dynamic;
					                            dateValidator.ErrorMessage = string.Format(MobileUtility.GetResource("DateValidatorText"), this.CurrentCulture.DateTimeFormat.ShortDatePattern);

					                            // Add the validator
                                                validatorCell.Controls.Add(dateValidator);  
                                            }
                                            else
                                            {
                                                if (dProperty != null)
                                                {
                                                    //Add the readonly label
                                                    this.AddReadOnlyControl(controlCell, dProperty.Value.date);
                                                }
                                            }
                                            break;

            case AttributeType.Money:       CrmMoneyProperty mProperty = (CrmMoneyProperty)property;

                                            if (!this._readOnly)
                                            {
                                                //Initialize and set the properties of the control
                                                TextBox mControl = new TextBox();
                                                mControl.ID = attrib.Name;

                                                if (mProperty != null)
                                                {
                                                    mControl.Text = mProperty.Value.formattedvalue;
                                                }

                                                //Add the control
                                                controlCell.Controls.Add(mControl);

                                                //Initialize the money metadata attribute based on this attribute
                                                MoneyAttributeMetadata mMeta = (MoneyAttributeMetadata)attrib;

					                            // Validator used to verify the money value inputted is in the correct format and range     
					                            CustomValidator moneyCustomValidator = new CustomValidator();
					                            moneyCustomValidator.ControlToValidate = attrib.Name;
					                            moneyCustomValidator.EnableClientScript = false;
					                            moneyCustomValidator.ServerValidate += new ServerValidateEventHandler(this.ValidateNumber);
					                            moneyCustomValidator.Display = ValidatorDisplay.Dynamic;
					                            moneyCustomValidator.ErrorMessage = String.Format(MobileUtility.GetResource("NumericValidatorText"), mMeta.MinValue.ToString(("N" + mMeta.Precision), this.CurrentCulture), mMeta.MaxValue.ToString(("N" + mMeta.Precision), this.CurrentCulture));

					                            // Add the validator
					                            validatorCell.Controls.Add(moneyCustomValidator);  
                                            }
                                            else
                                            {
                                                if (mProperty != null)
                                                {
                                                    //Add the read only label
                                                    this.AddReadOnlyControl(controlCell, mProperty.Value.formattedvalue);
                                                }
                                            }
                                            break;

            case AttributeType.Picklist:    PicklistProperty pProperty = (PicklistProperty)property;

                                            if (!this._readOnly)
                                            {
                                                string propertyValue = String.Empty;

                                                if (pProperty != null)
                                                {
                                                    propertyValue = pProperty.Value.Value.ToString(CultureInfo.InvariantCulture);
                                                }

                                                //Add the dropdownlist
                                                controlCell.Controls.Add(this.GetDropDownList(attrib, propertyValue));
                                            }
                                            else
                                            {
                                                if (pProperty != null)
                                                {
                                                    //Add the readonly label or link
                                                    this.AddReadOnlyControl(controlCell, pProperty.Value.name);
                                                }
                                            }
                                            break;

            case AttributeType.Boolean:     CrmBooleanProperty bProperty = (CrmBooleanProperty)property;

                                            if (!this._readOnly)
                                            {
                                                string propertyValue = String.Empty;

                                                if (bProperty != null)
                                                {
                                                    if (bProperty.Value.Value)
                                                    {
                                                        propertyValue = "1";
                                                    }
                                                    else
                                                    {
                                                        propertyValue = "0";
                                                    }
                                                }

                                                //Add the dropdownlist
                                                controlCell.Controls.Add(this.GetDropDownList(attrib, propertyValue));
                                            }
                                            else
                                            {
                                                if (bProperty != null)
                                                {
                                                    //Add the readonly label
                                                    this.AddReadOnlyControl(controlCell, bProperty.Value.name);
                                                }
                                            }
                                            break;

            case AttributeType.Owner:       OwnerProperty oProperty = (OwnerProperty)property;

                                            if (oProperty != null)
                                            {
                                                //Add the readonly label regardless if readonly or edit mode
                                                this.AddReadOnlyControl(controlCell, oProperty.Value.name);
                                            }

                                            break;

            case AttributeType.State:       StateProperty stProperty = (StateProperty)property;

                                            if (stProperty != null)
                                            {
                                                //Add the readonly label regardless if readonly or edit mode
                                                this.AddReadOnlyControl(controlCell, stProperty.Value);
                                            }
                                            break;
                                            
			case AttributeType.Status:		StatusProperty suProperty = (StatusProperty)property;

											if (suProperty != null)
											{
												//Add the readonly label
												this.AddReadOnlyControl(controlCell, suProperty.Value.name);
											}
											
											break;
											
			case AttributeType.PartyList:	DynamicEntityArrayProperty dyProperty = (DynamicEntityArrayProperty)property;
											DynamicEntity[] partyEntities = dyProperty.Value;
											
											string partyLabelValue = String.Empty;
											
											foreach(DynamicEntity pEntity in partyEntities)
											{
												LookupProperty pLookup = (LookupProperty)MobileUtility.GetDynamicEntityProperty(pEntity,"partyid");
												
												if(pLookup != null)
												{
													if(partyLabelValue.Length > 0)
													{
														partyLabelValue += ",";
													}
													partyLabelValue += pLookup.Value.name;
												}
											}
											
											this.AddReadOnlyControl(controlCell,partyLabelValue);
											
											break;
        }

        //If in edit mode, check for requirement
        if (!this._readOnly)
        {
            //Change the color of the label control based on requirement
            if (attrib.RequiredLevel == AttributeRequiredLevel.Required)
            {
                attribLabel.ForeColor = Color.Red;
                attribLabel.Text = "** " + attribLabel.Text;

                //Add Validator to the validator row
                validatorCell.Controls.Add(this.GetRequiredValidator(attrib));
			}                                                            
			else if(attrib.RequiredLevel == AttributeRequiredLevel.SystemRequired)
			{
				attribLabel.ForeColor = Color.Red;
				attribLabel.Text = "** " + attribLabel.Text;

				//Add Validator to the validator row if the attribute is not
				//owner id,state or status -- since these are readonly fields
				if(attrib.Type != AttributeType.Owner 
					&& attrib.Type != AttributeType.State 
					&& attrib.Type != AttributeType.Status)
				{
					validatorCell.Controls.Add(this.GetRequiredValidator(attrib));
				}
			}
			else if(attrib.RequiredLevel == AttributeRequiredLevel.Recommended)
			{
                attribLabel.ForeColor = Color.Blue;
                attribLabel.Text = "* " + attribLabel.Text;                                               
            }

        }



        //If the validator cell has controls add it to the table
        if (validatorCell.Controls.Count > 0)
        {
            validatorRow.Controls.Add(validatorCell);
            formTable.Controls.Add(validatorRow);
        }

        //Add the control row to the table
        controlRow.Controls.Add(controlCell);
        formTable.Controls.Add(controlRow);

    }

    /// <summary>
    /// This method will retrieve or create a dynamic entity for the current
    /// instance. The retrieval relies on the primarykey of the entity.
    /// </summary>
    private DynamicEntity GetCurrentEntity()
    {
        DynamicEntity de = null;

        //Get the field nodes for the entity from the config xml
        XmlNodeList fieldNodes = this.GetFieldNodes();

        //If the primary key has not been sent by the GET method then create a new dynamic entity
        if (_mPage.Request.QueryString[_eMeta.PrimaryKey] == null)
        {
            de = new DynamicEntity();

            //The property array count will be the same as the field nodes count
            de.Properties = new Property[0];

            //Set the name of the dynamic entity
            de.Name = _eMeta.Name;

            //Create the appropriate properties of the entity based off of the field nodes
            for (int i = 0; i < fieldNodes.Count; i++)
            {
                XmlAttribute nameAttribute = (XmlAttribute)fieldNodes[i].Attributes.GetNamedItem("name");

                if (nameAttribute != null)
                {
					AttributeMetadata attrib = MetadataCache.GetAttributeMetadata(_eMeta.Name, nameAttribute.Value);

                    if (attrib != null)
                    {
                        //Create the new property in the properties array of the dynamic entity
                        MobileUtility.CreateDynamicEntityProperty(de, attrib);
                    }
                }
            }
        }
        else
        {
            //Primary key has been sent by GET method, so we begin to fetch the instance

            // Create the Retrieve Target
            TargetRetrieveDynamic targetRetrieve = new TargetRetrieveDynamic();

            // Set the Target's Properties
            targetRetrieve.EntityName = _eMeta.Name;

            targetRetrieve.EntityId = new Guid(_mPage.Request.QueryString[_eMeta.PrimaryKey]);

            // Create the Request Object
            RetrieveRequest retrieve = new RetrieveRequest();

            // Indicates that the BusinessEntity should be retrieve as a DynamicEntity
            retrieve.ReturnDynamicEntities = true;

            // Set the Request Object's Properties
            retrieve.Target = targetRetrieve;

            //Create a new column set
            ColumnSet cSet = new ColumnSet();
			
			//The array list that the names of the columns will be added 
			ArrayList attributeList = new ArrayList();
			
            //The columns fetched will be the fields in the xml
            for (int i = 0; i < fieldNodes.Count; i++)
            {
                XmlAttribute nameAttribute = (XmlAttribute)fieldNodes[i].Attributes.GetNamedItem("name");

				attributeList.Add(nameAttribute.Value.ToLower());
            }

			//See if the list contains the name of the primary field
			//If it doesn't, add it
			if(!attributeList.Contains(_eMeta.PrimaryField))
			{
				attributeList.Add(_eMeta.PrimaryField);
			}
			
			//If the entity is not a note or an address then add the statecode
			//attribute if it does not exist. This will help prevent exceptions
			//in the security checks that need the state property value of the entity
			if(!_eMeta.Name.Equals(EntityName.annotation.ToString()) 
				&& !_eMeta.Name.Equals(EntityName.customeraddress.ToString()) 
				&& !_eMeta.Name.Equals(EntityName.competitor.ToString()))
			{ 
				if(!attributeList.Contains(STATE_COLUMN))
				{
					attributeList.Add(STATE_COLUMN);
				}
			}
			
			//Cycle through the list and populate the string array
			//that will be used as the attributes for the columnset
			string[] attributes = new string[attributeList.Count];
			
			for(int i = 0;i < attributeList.Count;i++)
			{
				attributes[i] = attributeList[i].ToString();
			}

            //Set the column set attributes
            cSet.Attributes = attributes;

            //Set the columnset to use for the retrieve
            retrieve.ColumnSet = cSet;

            // Execute the Request
            RetrieveResponse retrieved = (RetrieveResponse)_mPage.CurrentCrmService.Execute(retrieve);

            //Initialize the dynamic entity with the fetched business entity
            de = (DynamicEntity)retrieved.BusinessEntity;

            //Add the KeyProperty to the dynamic entity, the fetch does not include the keyproperty
            //This is necessary or else the save operation will fail
            this.AddKeyPropertyToDynamic(de, targetRetrieve.EntityId);
        }

        return de;
    }

    /// <summary>
    /// Method retrieves the node list of attribute fields from the config xml
    /// </summary>
    /// <returns></returns>
    public XmlNodeList GetFieldNodes()
    {
        //Get the attribute fields nodes from the xmldom
        XmlNode entityNode = _mPage.MobileConfig.SelectSingleNode("/MobileClient/Entities/Entity[@name = '" + _eMeta.Name + "']");

        XmlNodeList fieldNodes = entityNode.SelectNodes("Fields/Field");

        return fieldNodes;
    }

    /// <summary>
    /// Method to update or create an instance of an entity
    /// </summary>
    public void Save()
    {
        try
        {
            //Create a new dynamic entity to store the values from the form
            DynamicEntity de = new DynamicEntity();
            de.Properties = new Property[0];
            de.Name = _eMeta.Name;

            //See if the primary key value was passed in, if not then do
			//not add the value of the primary key to the entity
			//The key is only set on an existing instance that we're updating
            if (_mPage.Request.QueryString[_eMeta.PrimaryKey] != null)
            {
                //Add the key property to the dynamic entity so it can be properly saved
                this.AddKeyPropertyToDynamic(de, new Guid(_mPage.Request.QueryString[_eMeta.PrimaryKey]));
            }

            //Get the node list of fields from the config xml
            XmlNodeList fieldNodes = this.GetFieldNodes();

            //Set the properties of the dynamic entity to the values passed in from the form postback
            foreach (XmlNode fieldNode in fieldNodes)
            {
                XmlAttribute nameAttribute = (XmlAttribute)fieldNode.Attributes.GetNamedItem("name");

                if (nameAttribute != null)
                {
                    //Get the meta data of the attribute field
					AttributeMetadata attrib = MetadataCache.GetAttributeMetadata(_eMeta.Name, nameAttribute.Value);

                    if (attrib != null)
                    {
                        Property currentProperty = MobileUtility.GetDynamicEntityProperty(de, attrib.Name);

                        if (currentProperty == null)
                        {
                            //Create the property only if it's not of owner or state
                            if (attrib.Type != AttributeType.Owner && attrib.Type != AttributeType.State  
								&& attrib.Type != AttributeType.Status && attrib.Type != AttributeType.PrimaryKey)
                            {
                                MobileUtility.CreateDynamicEntityProperty(de, attrib);
                            }
                        }

                        //The string array to hold the values for the dynamic entity property
                        //The first index will be the actual value, the second is for an extra value
                        //requirement Example: type in customer
                        string[] valuesToSet = new string[3];

                        //Get the value for the property of the dynamic entity
                        if (attrib.Type == AttributeType.Lookup || attrib.Type == AttributeType.Customer)
                        {
                            //Get the value for the lookup from the value posted from the dropdownlist
							if (_mPage.Request.Form[_masterPagePrefix + attrib.Name + LOOKUP_PICKLIST_SUFFIX] != null)
                            {
								string[] lValues = _mPage.Request.Form[_masterPagePrefix + attrib.Name + LOOKUP_PICKLIST_SUFFIX].Split(',');

                                valuesToSet[0] = lValues[1];
                            }
                            else
                            {
                                //Get the hidden input field which stores the Guid value
                                HtmlInputHidden lGuidHidden = (HtmlInputHidden)this.FindControl(attrib.Name + LOOKUP_HIDDEN_SUFFIX);

                                //Get the value from the hidden field
                                valuesToSet[0] = lGuidHidden.Value;
                            }

                            //This is the entity name for the lookup assocation, used as type for customer
							valuesToSet[1] = _mPage.Request.Form[_masterPagePrefix + attrib.Name + LOOKUP_ENTITIES_SUFFIX];

                            //Get the control that has the name of the lookup entity instance
                            TextBox lookupNameBox = (TextBox)this.FindControl(attrib.Name);

                            //Set the third index to the name of the lookup entity instance
                            valuesToSet[2] = lookupNameBox.Text;

                            //If the dynamic entity is an additional address and the field
                            //is the parent id, then we will add the object type code of the 
                            //parent entity instance
                            if (de.Name.Equals(EntityName.customeraddress.ToString()) && attrib.Name.Equals("parentid"))
                            {
                                //Get the entity meta data of the instance the address will belong to
                                EntityMetadata lMeta = MetadataCache.GetEntityMetadata(valuesToSet[1]);

                                //Get the attribute metadata of the objecttypecode field
                                AttributeMetadata lookupAttribMeta = MetadataCache.GetAttributeMetadata(de.Name, "objecttypecode");

                                //Create the property for the object type code
                                MobileUtility.CreateDynamicEntityProperty(de, lookupAttribMeta);

                                string[] objectTypeValues = new string[2];
                                objectTypeValues[0] = lMeta.ObjectTypeCode.ToString();
                                objectTypeValues[1] = lMeta.Name;

                                //Set the value of the object type
                                MobileUtility.SetDynamicEntityPropertyValue(_mPage, de, lookupAttribMeta.Name, objectTypeValues);
                            }
                        }
                        else
                        {
							valuesToSet[0] = _mPage.Request.Form[_masterPagePrefix + attrib.Name.ToLower()];
                        }

                        //In order to set the value of the property, the attribute
						//can not be of type Primary Key. The KeyProperty for the entity instance
						//will already be sent in if updating an existing entity instance.
                        if (attrib.Type != AttributeType.PrimaryKey)
                        {
                            //Set the dymamic entity property value
                            MobileUtility.SetDynamicEntityPropertyValue(_mPage, de, attrib.Name.ToLower(), valuesToSet);
                        }
                    }
                }
            }

            //If the guid is empty - create a new instance
            if (_mPage.Request.QueryString[_eMeta.PrimaryKey] == null)
            {
                Guid newEntityId = (Guid)_mPage.CurrentCrmService.Create(de);
            }
            else
            {
                //Update the instance
                _mPage.CurrentCrmService.Update(de);
            }

            //Redirect to the entity home page if successful create or update
			string referencingEntity = HttpUtility.HtmlEncode(_mPage.Request.QueryString["referencingentity"]);
			string referencingRecordName = HttpUtility.HtmlEncode(_mPage.Request.QueryString["referencingrecordname"]);
			string referencingId = HttpUtility.HtmlEncode(_mPage.Request.QueryString["referencingid"]);		

			// If we are on a related entity, go back to the entity we started from
			if ((referencingEntity != null && referencingEntity.Length != 0) &&
				(referencingRecordName != null && referencingRecordName.Length != 0) &&
				(referencingId != null && referencingId.Length != 0))
			{
				Guid referencingGuid = new Guid(referencingId);
				EntityMetadata referencingMetdata = MetadataCache.GetEntityMetadata(referencingEntity);
				_mPage.Response.Redirect("EntityReadOnly.aspx?entity=" + referencingEntity + "&" + referencingMetdata.PrimaryKey + "=" + referencingGuid.ToString("b"));
			}
			else
			{
				_mPage.Response.Redirect("EntityHome.aspx?" + ENTITY_REQUEST_INDEX + "=" + _eMeta.Name);
            }
        }
        catch (Exception e)
        {
            _mPage.Response.Write(e.StackTrace);
        }
    }
    
    /// <summary>
    /// This method adds the KeyProperty to the dynamic entity
    /// </summary>
    private void AddKeyPropertyToDynamic(DynamicEntity de, Guid id)
    {
        //New Property array
        Property[] newProperties = new Property[(de.Properties.Length + 1)];

        //Create the key property
        KeyProperty key = new KeyProperty();
        key.Value = new Key();
        key.Value.Value = id;
        key.Name = _eMeta.PrimaryKey;

        //Make the first index of the properties array to the keyproperty		
        newProperties[0] = key;

        //Make the rest of the property array equal to the dynamic entity properties
        for (int i = 0; i < de.Properties.Length; i++)
        {
            newProperties[(i + 1)] = de.Properties[i];
        }

        //Set the dynamic entity properties to the new Property array
        de.Properties = newProperties;
    }

    /// <summary>
    /// Event handler method for a save
    /// </summary>
    public void Save_Click(Object sender, EventArgs e)
    {
        //If the validators all validate, then save
        if (_mPage.IsValid)
        {
            this.Save();
        }
    }

    /// <summary>
    /// This is the method that validates the lookup values
    /// </summary>
    public void ValidateLookup(object source, ServerValidateEventArgs e)
    {
        //Get the validator
        CustomValidator validator = (CustomValidator)source;

        //Get the control that has the lookup primary field name
        Control control = this.FindControl(validator.ControlToValidate);
        DropDownList entityList = (DropDownList)this.FindControl(validator.ControlToValidate + LOOKUP_ENTITIES_SUFFIX);

		EntityMetadata targetEntityMeta = MetadataCache.GetEntityMetadata(entityList.SelectedValue);

        if (control is TextBox)
        {
            TextBox tControl = (TextBox)control;

			string lookupPicklistValue = _mPage.Request.Form[_masterPagePrefix + tControl.ID + LOOKUP_PICKLIST_SUFFIX];
            HtmlInputHidden lookupGuid = (HtmlInputHidden)this.FindControl(tControl.ID + LOOKUP_HIDDEN_SUFFIX);

            if (lookupPicklistValue != null)
            {
                string[] lookupParams = lookupPicklistValue.Split(',');

                tControl.Text = lookupParams[0];
                lookupGuid.Value = lookupParams[1];

				tControl.Visible = true;
				e.IsValid = true;
            }
			else
			{
				//Get the business entities of the target lookup entity
				BusinessEntity[] targetEntities = MobileUtility.GetDynamicBusinessEntities(_mPage, targetEntityMeta, tControl.Text);

				//Evaluate the number of instances for validation
				switch (targetEntities.Length)
				{
					case 0:		//There were no instances available for the primary field value entered
						validator.Text = MobileUtility.GetResource("NoMatchLookupValidatorText");
						e.IsValid = false;
						tControl.Visible = true;
						break;

					case 1:		//There was exactly one match found for the primary field value entered
						e.IsValid = true;
						tControl.Visible = true;

						DynamicEntity targetEntity = (DynamicEntity)targetEntities[0];

						//Set the value of the textbox and the hidden guid value to the primary values of the entity
						tControl.Text = MobileUtility.GetDynamicEntityPropertyValue(targetEntity, targetEntityMeta.PrimaryField);
						lookupGuid.Value = MobileUtility.GetDynamicEntityPropertyValue(targetEntity, targetEntityMeta.PrimaryKey);

						break;

					default:	//There is more than 1 instance available to choose as the lookup value

						//Get the parent of the control
						Control targetParent = tControl.Parent;

						//Hide the textbox with the primary field name
						tControl.Visible = false;

						//Create a dropdownlist that the user can use to select the lookup instance
						DropDownList instanceList = new DropDownList();

						//Populate the dropdownlist
						for (int i = 0; i < targetEntities.Length; i++)
						{
							DynamicEntity entity = (DynamicEntity)targetEntities[i];
							string listValue = MobileUtility.GetDynamicEntityPropertyValue(entity, targetEntityMeta.PrimaryField);

							instanceList.Items.Add(new ListItem(HttpUtility.HtmlEncode(listValue), HttpUtility.HtmlAttributeEncode(listValue + "," + MobileUtility.GetDynamicEntityPropertyValue(entity, targetEntityMeta.PrimaryKey))));
						}

						instanceList.ID = tControl.ID + LOOKUP_PICKLIST_SUFFIX;

						//Add the dropdownlist to the parent control of the textbox
						targetParent.Controls.Add(instanceList);

						validator.Text = MobileUtility.GetResource("MoreThanOneLookupValidatorText");

						e.IsValid = false;
						break;

				}
			}

        }
    }

    /// <summary>
    /// This method will evaluate the value being passed in. If the value matches a url
    /// regular expression then the control that is added to the form will be a link. If
    /// it does not match the regular expression then it will become a label. We use the 
    /// regular expression because there is no way of determining if a string is a url from
    /// the metadata.
    /// </summary>
    private void AddReadOnlyControl(Control parent, string value)
    {
        //The regular expression to match against the value
        Regex expression = new Regex("(^|[ \t\r\n])((ftp|http|https):(([A-Za-z0-9$_.+!*(),;/?:@&~=-])|%[A-Fa-f0-9]{2}){2,}(#([a-zA-Z0-9][a-zA-Z0-9$_.+!*(),;/?:@&~=%-]*))?([A-Za-z0-9$_+!*();/?:~-]))");

        //If there is a match then create the link and add it to the parent control
        // Invalid Picklists will pass in NULL, so we need to handle that
        if (value != null && expression.IsMatch(value))
        {
            HyperLink webLink = new HyperLink();
            webLink.Text = HttpUtility.HtmlEncode(value);
            webLink.NavigateUrl = HttpUtility.HtmlEncode(value);
            
            webLink.CssClass = "label";

            parent.Controls.Add(webLink);
        }
        else
        {
            //If there is not a match, then create the label and add it to the parent control
            Label labelControl = new Label();

            labelControl.Text = HttpUtility.HtmlEncode(value);
            
            labelControl.CssClass = "label";

            parent.Controls.Add(labelControl);
        }
    }

    /// <summary>
    /// This method returns the dropdownlist for the edit form for similar types
    /// </summary>
    private DropDownList GetDropDownList(AttributeMetadata attrib, string value)
    {
        //Create the dropdownlist
        DropDownList pControl = new DropDownList();
        pControl.ID = attrib.Name;

        //Populate the dropdownlist
        
        if(attrib is PicklistAttributeMetadata)
        {
			PicklistAttributeMetadata pMeta = (PicklistAttributeMetadata)attrib;
			Option[] options = pMeta.Options;

			foreach (Option option in options)
			{
				pControl.Items.Add(new ListItem(HttpUtility.HtmlEncode(option.Description), HttpUtility.HtmlAttributeEncode(option.OptionValue.ToString(CultureInfo.InvariantCulture))));
			}
		}
		else
		{
			throw new Exception();
		}
		
        //Set the selected value of the dropdownlist
        pControl.SelectedValue = value;


        return pControl;
    }

    /// <summary>
    /// This method will add the appropriate edit and readonly controls
    /// for lookup and customer datatypes
    /// </summary>
    /// <param name="controlCell">The cell that contains the control</param>
    /// <param name="validatorCell">The cell that contains the validator</param>
    /// <param name="property">The LookupProperty or CustomerProperty object</param>
    /// <param name="attrib">The attribute metadata</param>
    /// <param name="formTable">The table of controls of this form which rows are appended</param>
    private void AddLookupControl(Control controlCell,
                                    Control validatorCell,
                                    Property property,
                                    AttributeMetadata attrib,
                                    Control formTable)
    {
        //Get entity types associated with the lookup
        string lookupEntities = MobileUtility.GetRelatedEntityTypes(_eMeta, attrib.Name);

        string[] lEntityTypes = lookupEntities.Split(',');

        if (!this._readOnly)
        {
            TextBox lControl = new TextBox();
            lControl.ID = attrib.Name;

            //Add the textbox control
            controlCell.Controls.Add(lControl);

            //Create the hidden field to store the guid of the lookup
            HtmlInputHidden lookupGuid = new HtmlInputHidden();
            lookupGuid.ID = attrib.Name + LOOKUP_HIDDEN_SUFFIX;

            //Add the hidden input control
            controlCell.Controls.Add(lookupGuid);

            //Create the entities dropdownlist
            DropDownList lookupEntitiesDropDown = new DropDownList();
            lookupEntitiesDropDown.ID = attrib.Name + LOOKUP_ENTITIES_SUFFIX;

            //Populate the entities dropdownlist
            for (int i = 0; i < lEntityTypes.Length; i++)
            {
                string displayName = lEntityTypes[i];

				EntityMetadata lookupMeta = MetadataCache.GetEntityMetadata(lEntityTypes[i]);

                if (lookupMeta != null)
                {
                    displayName = lookupMeta.DisplayName;
                }

                lookupEntitiesDropDown.Items.Add(new ListItem(displayName, lEntityTypes[i]));
            }

            //Set the value of the hidden control and text of the textbox
            if (property != null)
            {
                if (property is LookupProperty)
                {
                    LookupProperty lProperty = (LookupProperty)property;
                    
                    lookupGuid.Value = lProperty.Value.Value.ToString();

					if(lProperty.Value.name == null && lProperty.Value.type == null)
					{
						this.PopulateLookupProperty(lProperty,lEntityTypes);
					}
					
					if(lProperty.Value.type == null)
					{
						//The type property of the LookupProperty is null, so take the value of entity related from
						//previous fetch to select the appropriate index
						lookupEntitiesDropDown.SelectedValue = lEntityTypes[0];
					}
					else
					{
						lookupEntitiesDropDown.SelectedValue = lProperty.Value.type;
					}

					lControl.Text = lProperty.Value.name;
                }
                else if (property is CustomerProperty)
                {
                    CustomerProperty lProperty = (CustomerProperty)property;

                    lControl.Text = lProperty.Value.name;
                    lookupGuid.Value = lProperty.Value.Value.ToString();

                    //Select the appropriate index
                    lookupEntitiesDropDown.SelectedValue = lProperty.Value.type;
                }
                else
                {
                    //Throw an exception if the property is not a LookupProperty or a CustomerProperty
                    throw new Exception();
                }
            }

            //Add the row that will contain the name of the entity for lookup
            HtmlTableRow row = new HtmlTableRow();
            HtmlTableCell cell = new HtmlTableCell();

            //Add the entity list row to the form table
            cell.Controls.Add(lookupEntitiesDropDown);
            row.Controls.Add(cell);
            formTable.Controls.Add(row);

            //Add the validator for the lookup
            CustomValidator lValidator = new CustomValidator();      
            lValidator.ControlToValidate = attrib.Name;
            lValidator.EnableClientScript = false;
            lValidator.ServerValidate += new ServerValidateEventHandler(this.ValidateLookup);
            lValidator.Display = ValidatorDisplay.Dynamic;
            validatorCell.Controls.Add(lValidator);

        }
        else
        {
            //In readonly the the lookups will be links to their readonly page
            HyperLink lControl = new HyperLink();
            lControl.ID = attrib.Name;

            if (property != null)
            {
                //String to hold the value of the related entity name
                string eName = String.Empty;

                //String to hold the guid value of the related entity instance
                string lGuid = String.Empty;

                //Initialize the string values if the property is LookupProperty
                if (property is LookupProperty)
                {
                    LookupProperty lProperty = (LookupProperty)property;

					if (lProperty.Value.name == null && lProperty.Value.type == null)
					{
						this.PopulateLookupProperty(lProperty,lEntityTypes);
					}

                    lControl.Text = lProperty.Value.name;

                    if(lProperty.Value.type == null)
                    {
						//The eName value is the value of the entity fetched before
						//because the type property of the LookupProperty is null
						eName = lEntityTypes[0];
					}
					else
					{
						eName = lProperty.Value.type;
					}

                    lGuid = lProperty.Value.Value.ToString();
                }
                else if (property is CustomerProperty)
                {
                    CustomerProperty lProperty = (CustomerProperty)property;

                    lControl.Text = lProperty.Value.name;

                    eName = lProperty.Value.type;

                    lGuid = lProperty.Value.Value.ToString();
                }
                else
                {
                    //Throw an exception if the property is not a LookupProperty or a CustomerProperty
                    throw new Exception();
                }

                //Get the metadata of the related entity so that we can complete the navigation url
				EntityMetadata lMeta = MetadataCache.GetEntityMetadata(eName);

                if (MobileUtility.IsEntityEnabled(eName) && MobileUtility.DoesUserHaveRightsToRecord(eName, new Guid(lGuid), AccessRights.ReadAccess))
                {
                    //TODO: Need to set the naviagation url with the appropriate entity
                    lControl.NavigateUrl = "EntityReadOnly.aspx?entity="
                                            + eName
                                            + "&"
                                            + lMeta.PrimaryKey
                                            + "="
                                            + lGuid;
                }
            }
            
            lControl.CssClass = "label";

            //Add the link control
            controlCell.Controls.Add(lControl);
        }
    }

	/// <summary>
	/// This validator is used for all number fields and ensures that the value entered is in the correct 
	/// format and within the bounds of the attribute.
	/// </summary>
    public void ValidateNumber(object source, ServerValidateEventArgs e)
    {
		// Retrieve the control by referencing it from the validator
        CustomValidator validator = source as CustomValidator;        
        Control control = this.FindControl(validator.ControlToValidate);
        TextBox textBox = control as TextBox;

		// If any parse fails - the value entered is not in the correct format
        try
        {
			// Retrieve attribute metadata about attribute of control
            AttributeMetadata attribute = MetadataCache.GetAttributeMetadata(this.CurrentEntityMetadata.Name, textBox.ID);
            
            switch (attribute.Type)
            {
                case AttributeType.Integer:
					// Parse value of the text box using the format of the current culture (which is set by the Organization)
					int intValue = Int32.Parse(textBox.Text, NumberStyles.Number, this.CurrentCulture);

                    IntegerAttributeMetadata intMetadata = attribute as IntegerAttributeMetadata;

					// If the value is within the bounds of the Min and Max the value is valid
                    if (intValue >= intMetadata.MinValue && intValue <= intMetadata.MaxValue)
                    {
                        e.IsValid = true;
                    }
                    else
                    {
                        e.IsValid = false;
                    }
                    
                    break;
                    
                case AttributeType.Float:
					float floatValue = float.Parse(textBox.Text, NumberStyles.Float, this.CurrentCulture);
					
					FloatAttributeMetadata floatMetadata = attribute as FloatAttributeMetadata;

					if (floatValue >= floatMetadata.MinValue && floatValue <= floatMetadata.MaxValue)
					{
						// This split will usually yield two results
						string[] splitFloatValue = floatValue.ToString(CultureInfo.InvariantCulture).Split('.');
						
						// For numbers like 1.5
						if (splitFloatValue.Length > 1)
						{
							// Check that the length of the decimal digits are within the bounds of the precision
							if (splitFloatValue[1].Length >= 0 && splitFloatValue[1].Length <= floatMetadata.Precision)
							{
								e.IsValid = true;
							}					
							else
							{
								e.IsValid = false;
							}	
						}
						// For numbers like .5 (if the decimal separator is found in element 0, there are no whole digits)
						else if (splitFloatValue[0].IndexOf(this.CurrentCulture.NumberFormat.NumberDecimalSeparator) > -1)
						{
							// Check that the length of the decimal digits are within the bounds of the precision
							if (splitFloatValue[0].Length >= 0 && splitFloatValue[0].Length <= floatMetadata.Precision)
							{
								e.IsValid = true;
							}
							else
							{
								e.IsValid = false;
							}	
						}
						else
						{
							e.IsValid = true;
						}						 						
					}
					else
					{
						e.IsValid = false;
					}
					
					break;
					
				case AttributeType.Money:
					double doubleValue = double.Parse(textBox.Text, NumberStyles.Currency, this.CurrentCulture);
					
					MoneyAttributeMetadata moneyMetadata = attribute as MoneyAttributeMetadata;

					if (doubleValue >= moneyMetadata.MinValue && doubleValue <= moneyMetadata.MaxValue)
					{
						string[] splitDoubleValue = doubleValue.ToString(CultureInfo.InvariantCulture).Split('.');

						if (splitDoubleValue.Length > 1)
						{
							if (splitDoubleValue[1].Length >= 0 && splitDoubleValue[1].Length <= moneyMetadata.Precision)
							{
								e.IsValid = true;
							}
							else
							{
								e.IsValid = false;
							}
						}
						else if (splitDoubleValue[0].IndexOf(this.CurrentCulture.NumberFormat.CurrencyDecimalSeparator) > -1)
						{
							if (splitDoubleValue[0].Length >= 0 && splitDoubleValue[0].Length <= moneyMetadata.Precision)
							{
								e.IsValid = true;
							}
							else
							{
								e.IsValid = false;
							}
						}
						else
						{
							e.IsValid = true;
						}		
					}
					else
					{
						e.IsValid = false;
					}
					
					break;					
					
                default:
                    e.IsValid = false;
                    break;
            }            
        }
        catch
        {
            e.IsValid = false;
        }        
    }

	/// <summary>
	/// Validates a date by using the org's culture format
	/// </summary>
    public void ValidateDate(object source, ServerValidateEventArgs e)
    {
        // Retrieve the control by referencing it from the validator
        CustomValidator validator = source as CustomValidator;
        Control control = this.FindControl(validator.ControlToValidate);
        TextBox textBox = control as TextBox;

        try
        {
			// The date must match the format exactly to avoid any errors
            DateTime.Parse(textBox.Text, this.CurrentCulture);

			// If parse works, we know the date is fine
            e.IsValid = true;
        }
        catch
        {
            e.IsValid = false;
        }
    }
    
    /// <summary>
    /// This method will search for a dynamic enity of the entitie names passed in
	/// and the Guid value from the lookup property passed in. If a match is found,
	/// then the name and type values of the lookup are set with the values
	/// of the dynamic entity found. After the values are set the method returns. 
    /// </summary>
    /// <param name="lProperty">Existing lookup property</param>
    /// <param name="entities">List of possible entity names that the lookup could be.</param>
    private void PopulateLookupProperty(LookupProperty lProperty,string[] entities)
    {
		foreach(string entityName in entities)
		{
			EntityMetadata targetEntityMeta = MetadataCache.GetEntityMetadata(entityName);
			
			// Create the Request Object
			RetrieveMultipleRequest retrieve = new RetrieveMultipleRequest();
			retrieve.ReturnDynamicEntities = true;

			//Create the query helper for setting the query
			QueryExpressionHelper helper = new QueryExpressionHelper(targetEntityMeta.Name);
			helper.Columns.AddColumns(targetEntityMeta.PrimaryField, targetEntityMeta.PrimaryKey);
			helper.Criteria.Conditions.AddCondition(targetEntityMeta.PrimaryKey, ConditionOperator.Equal,lProperty.Value.Value.ToString());

			//Set the query
			retrieve.Query = helper.Query;

			// Execute the Request
			RetrieveMultipleResponse retrieved = (RetrieveMultipleResponse)_mPage.CurrentCrmService.Execute(retrieve);
			BusinessEntity[] bEntities = retrieved.BusinessEntityCollection.BusinessEntities;
		
			if(bEntities.Length == 1)
			{
				DynamicEntity de = (DynamicEntity)bEntities[0];
				
				lProperty.Value.name = MobileUtility.GetDynamicEntityPropertyValue(de,targetEntityMeta.PrimaryField);
				lProperty.Value.type = targetEntityMeta.Name;
				return;
			}
		}
    }
    
    /// <summary>
    /// This method adds the edit link to the form
    /// </summary>
    private void AddEditLinkButton()
    {
		LinkButton editButton = new LinkButton();
		editButton.Text = MobileUtility.GetResource("EditButtonText");
		editButton.ID = "EditButton";
		editButton.Style.Add("font-weight ", "bold");

		// If we are on a related entity page, we must pass all the querystring values to the edit page as well
		string referencingEntity = HttpUtility.HtmlEncode(_mPage.Request.QueryString["referencingentity"]);
		string referencingRecordName = HttpUtility.HtmlEncode(_mPage.Request.QueryString["referencingrecordname"]);
		string referencingId = HttpUtility.HtmlEncode(_mPage.Request.QueryString["referencingid"]);		
				
		// Set the edit button to post cross page
		if ((referencingEntity != null && referencingEntity.Length != 0) &&
			(referencingRecordName != null && referencingRecordName.Length != 0) &&
			(referencingId != null && referencingId.Length != 0))
		{
			Guid referencingGuid = new Guid(referencingId);
			
			editButton.PostBackUrl = "EntityEdit.aspx?entity="
								+ _eMeta.Name
								+ "&"
								+ _eMeta.PrimaryKey
								+ "="
								+ HttpUtility.HtmlEncode(_mPage.Request.QueryString[_eMeta.PrimaryKey])
								+ "&referencingentity="
								+ referencingEntity
							    + "&referencingrecordname="
								+ referencingRecordName
							    + "&referencingid="
								+ referencingGuid.ToString("b");
		}
		else
		{
			editButton.PostBackUrl = "EntityEdit.aspx?entity="
								+ _eMeta.Name
								+ "&"
								+ _eMeta.PrimaryKey
								+ "="
								+ HttpUtility.HtmlEncode(_mPage.Request.QueryString[_eMeta.PrimaryKey]);
		}

		//Add the edit button to the this form
		this.Controls.Add(editButton);

		// Add HR after Edit Button/Message
		FormBar formBar = new FormBar();
		this.Controls.Add(formBar);
    }
    
    private RequiredFieldValidator GetRequiredValidator(AttributeMetadata attrib)
    {
		//Create the validator for the field
		RequiredFieldValidator validator = new RequiredFieldValidator();
		validator.ControlToValidate = attrib.Name;
		validator.Text = MobileUtility.GetResource("RequiredValidatorText");
		validator.Display = ValidatorDisplay.Dynamic;
		validator.EnableClientScript = false;
		
		return validator;
    }
}
