// =======================================================================
//  File: MobileUtility.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Threading;
using System.Globalization;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

using Microsoft.Crm.Sandbox.Mobile.Helpers;
using Microsoft.Crm.Sandbox.Mobile.Helpers.Collection;

/// <summary>
/// This is the utility that can be used for methods that are used frequently among
/// the different pages. Many of the methods here deal specifically with dynamic entities.
/// </summary>
public class MobileUtility
{
	private const int VALUE_INDEX = 0;
	private const int TYPE_INDEX = 1;
	private const int NAME_INDEX = 2;
	
	public MobileUtility(){}

	//This method will get the value of the specific property of the dynamic entity
	public static string GetDynamicEntityPropertyValue(DynamicEntity entity,string searchProperty)
	{
		for (int i = 0; i < entity.Properties.Length; i++)
		{

			if (entity.Properties[i] != null)
			{
				//Check to see if the property equals the search term passed in
				if (entity.Properties[i].Name.ToLower() == searchProperty)
				{
					//Evaluate the Property type and return the appropriate value
					if(entity.Properties[i] is KeyProperty)
					{
						return ((KeyProperty)entity.Properties[i]).Value.Value.ToString();
					}
					else if(entity.Properties[i] is StringProperty)
					{
						return ((StringProperty)entity.Properties[i]).Value;
					}
                    else if (entity.Properties[i] is StateProperty)
                    {
                        return ((StateProperty)entity.Properties[i]).Value;
                    }					
				}
			}
		}

		return String.Empty;
	}

	/// <summary>
	/// This method returns the property which name is equal to the search term
	/// </summary>
	public static Property GetDynamicEntityProperty(DynamicEntity entity, string searchProperty)
	{	
		for (int i = 0; i < entity.Properties.Length; i++)
		{

			if(entity.Properties[i] != null)
			{	
				//If the name equals the term, return the property
				if (entity.Properties[i].Name.ToLower() == searchProperty)
				{
					return entity.Properties[i];
				}
			}
		}

		return null;
	}
	
	/// <summary>
	/// This method will return a comma delimited string of related
	/// entity names.
	/// </summary>
	public static string GetRelatedEntityTypes(EntityMetadata eMeta,string attributeName)
	{
		RelationshipMetadata[] relationship = eMeta.ReferencesFrom;
		string entities = String.Empty;
		
		for (int i = 0; i < relationship.Length; i++)
		{

			if (relationship[i].ReferencingAttribute.Equals(attributeName))
			{
				if(entities.Length > 0)
				{
					entities += ",";
				}
				
				entities += relationship[i].ReferencedEntity.ToString();
			}

		}

		return entities;
	}

	/// <summary>
	/// This method will set the values of the property of the dynamic entity
	/// which the property name is equivalent to searchProperty value
	/// </summary>
	public static void SetDynamicEntityPropertyValue(Mobile mPage,DynamicEntity entity, string searchProperty,string[] values)
	{
		User user = UserCache.GetCurrentUser();
		Organization org = OrganizationCache.GetOrganization(user);
		
		//TODO: Allow nulling of controls		
		
		for (int i = 0; i < entity.Properties.Length; i++)
		{

			if (entity.Properties[i].Name.ToLower() == searchProperty)
			{

				if (entity.Properties[i] is KeyProperty)
				{
					KeyProperty kProperty = (KeyProperty)entity.Properties[i];
					kProperty.Value.Value = new Guid(values[VALUE_INDEX]);
					entity.Properties[i] = kProperty;
				}
				else if (entity.Properties[i] is LookupProperty)
				{
					LookupProperty lProperty = (LookupProperty)entity.Properties[i];

					lProperty.Value.type = values[TYPE_INDEX];

					if (values[NAME_INDEX] != null && values[NAME_INDEX].Length != 0)
					{
						lProperty.Value.name = values[NAME_INDEX];
						lProperty.Value.Value = new Guid(values[VALUE_INDEX]);
					}
					else
					{
						lProperty.Value.IsNull = true;
						lProperty.Value.IsNullSpecified = true;
					}
					
					entity.Properties[i] = lProperty;
				}
				else if(entity.Properties[i] is CustomerProperty)
				{
					CustomerProperty cProperty = (CustomerProperty)entity.Properties[i];

					cProperty.Value.type = values[TYPE_INDEX];	

					if (values[NAME_INDEX] != null && values[NAME_INDEX].Length != 0)
					{						
						cProperty.Value.name = values[NAME_INDEX];		
						cProperty.Value.Value = new Guid(values[VALUE_INDEX]);
					}
					else
					{
						cProperty.Value.IsNull = true;
						cProperty.Value.IsNullSpecified = true;			
					}					
					
					entity.Properties[i] = cProperty;
				}
				else if (entity.Properties[i] is CrmDateTimeProperty)
				{
					CrmDateTimeProperty dProperty = (CrmDateTimeProperty)entity.Properties[i];					
					
					// If the value is blank, the parse will fail, so we will just clear the value out
					if (values[VALUE_INDEX] != null && values[VALUE_INDEX].Length != 0)
					{
						dProperty.Value.Value = DateTime.Parse(values[VALUE_INDEX], org.OrganizationCulture).ToString(CultureInfo.InvariantCulture);
					}
					else
					{
						dProperty.Value.Value = string.Empty;
					}	
					
					entity.Properties[i] = dProperty;
				}
				else if (entity.Properties[i] is CrmNumberProperty)
				{
					CrmNumberProperty nProperty = (CrmNumberProperty)entity.Properties[i];

					// If the value is blank, the parse will fail, so we will just clear the value out
					if (values[VALUE_INDEX] != null && values[VALUE_INDEX].Length != 0)
					{
						nProperty.Value.Value = int.Parse(values[VALUE_INDEX], NumberStyles.Number, org.OrganizationCulture);					
					}
					else
					{
						nProperty.Value.IsNull = true;
						nProperty.Value.IsNullSpecified = true;						
					}	
					
					entity.Properties[i] = nProperty;			
				}
				else if (entity.Properties[i] is CrmFloatProperty)
				{
					CrmFloatProperty fProperty = (CrmFloatProperty)entity.Properties[i];

					// If the value is blank, the parse will fail, so we will just clear the value out
					if (values[VALUE_INDEX] != null && values[VALUE_INDEX].Length != 0)
					{
						fProperty.Value.Value = float.Parse(values[VALUE_INDEX], NumberStyles.Float, org.OrganizationCulture);
					}
					else
					{
						fProperty.Value.IsNull = true;
						fProperty.Value.IsNullSpecified = true;
					}
					
					entity.Properties[i] = fProperty;
				}
				else if (entity.Properties[i] is CrmMoneyProperty)
				{
					CrmMoneyProperty mProperty = (CrmMoneyProperty)entity.Properties[i];

					// If the value is blank, the parse will fail, so we will just clear the value out
					if (values[VALUE_INDEX] != null && values[VALUE_INDEX].Length != 0)
					{
						mProperty.Value.Value = decimal.Parse(values[VALUE_INDEX], NumberStyles.Currency, org.OrganizationCulture);
					}
					else
					{
						mProperty.Value.IsNull = true;
						mProperty.Value.IsNullSpecified = true;
					}
					
					entity.Properties[i] = mProperty;
				}
				else if (entity.Properties[i] is PicklistProperty)
				{
					PicklistProperty pProperty = (PicklistProperty)entity.Properties[i];
					pProperty.Value.Value = Int32.Parse(values[VALUE_INDEX]);
					
					if(values[TYPE_INDEX] != null)
					{
						pProperty.Value.name = values[TYPE_INDEX];
					}
					entity.Properties[i] = pProperty;
					
				}
				else if(entity.Properties[i] is CrmBooleanProperty)
				{
					CrmBooleanProperty bProperty = (CrmBooleanProperty)entity.Properties[i];
					
					if(values[VALUE_INDEX].Equals("1"))
					{
						bProperty.Value.Value = true;
					}
					else
					{
						bProperty.Value.Value = false;
					}
					entity.Properties[i] = bProperty;
				}
				else
				{
					StringProperty sProperty = (StringProperty)entity.Properties[i];
					sProperty.Value = values[VALUE_INDEX];
					entity.Properties[i] = sProperty;
				}

				break;

			}

		}
	}
	
	/// <summary>
	/// This method will add a Property to the properties of the the dynamic entity
	/// based on the attribute metadata
	/// </summary>
	public static void CreateDynamicEntityProperty(DynamicEntity de,AttributeMetadata attrib)
	{
		//New Property array
		Property[] newProperties = new Property[(de.Properties.Length + 1)];
		
		switch(attrib.Type)
		{
			case AttributeType.String:		StringProperty sProperty = new StringProperty();
											sProperty.Name = attrib.Name;
											newProperties[0] = sProperty;
											break;
											
			case AttributeType.Picklist:	PicklistProperty pProperty = new PicklistProperty();
											pProperty.Name = attrib.Name;
											pProperty.Value = new Picklist();
											newProperties[0] = pProperty;
											break;

			case AttributeType.Boolean:		CrmBooleanProperty bProperty = new CrmBooleanProperty();
											bProperty.Name = attrib.Name;
											bProperty.Value = new CrmBoolean();
											newProperties[0] = bProperty;
											break;
											
			case AttributeType.Money:		CrmMoneyProperty mProperty = new CrmMoneyProperty();
											mProperty.Name = attrib.Name;
											mProperty.Value = new CrmMoney();
											newProperties[0] = mProperty;
											break;
											
			case AttributeType.Integer:		CrmNumberProperty nProperty = new CrmNumberProperty();
											nProperty.Name = attrib.Name;
											nProperty.Value = new CrmNumber();
											newProperties[0] = nProperty;
											break;
											
			case AttributeType.Lookup:		LookupProperty lProperty = new LookupProperty();
											lProperty.Name = attrib.Name;
											lProperty.Value = new Lookup();
											newProperties[0] = lProperty;
											break;
											
			case AttributeType.Customer:	CustomerProperty cProperty = new CustomerProperty();
											cProperty.Name = attrib.Name;
											cProperty.Value = new Customer();
											newProperties[0] = cProperty;
											break;
											
			case AttributeType.Float:		CrmFloatProperty fProperty = new CrmFloatProperty();
											fProperty.Name = attrib.Name;
											fProperty.Value = new CrmFloat();
											newProperties[0] = fProperty;
											break;
											
			case AttributeType.DateTime:	CrmDateTimeProperty dProperty = new CrmDateTimeProperty();
											dProperty.Name = attrib.Name;
											dProperty.Value = new CrmDateTime();
											newProperties[0] = dProperty;
											break;
		}

		//Make the rest of the property array equal to the dynamic entity properties
		for (int i = 0; i < de.Properties.Length; i++)
		{
			newProperties[(i + 1)] = de.Properties[i];
		}

		//Set the dynamic entity properties to the new Property array
		de.Properties = newProperties;
	}

	/// <summary>
	/// This method returns a BusinessEntity collection based off of the
	/// primary field value.
	/// </summary>
	public static BusinessEntity[] GetDynamicBusinessEntities(Mobile mPage,EntityMetadata targetEntityMeta, string primaryValue)
	{
		// Create the Request Object
		RetrieveMultipleRequest retrieve = new RetrieveMultipleRequest();
		retrieve.ReturnDynamicEntities = true;

		//Create the query helper for setting the query
		QueryExpressionHelper helper = new QueryExpressionHelper(targetEntityMeta.Name);
		helper.Columns.AddColumns(targetEntityMeta.PrimaryField);
		helper.Criteria.Conditions.AddCondition(targetEntityMeta.PrimaryField, ConditionOperator.Like, primaryValue + "%");

		//Get the maximum number of returned instances specified in the web.config
		helper.PageInfo.Count = mPage.GetMaxInstancesReturned();
		helper.PageInfo.PageNumber = 1;

		//Set the query
		retrieve.Query = helper.Query;

		// Execute the Request
		RetrieveMultipleResponse retrieved = (RetrieveMultipleResponse)mPage.CurrentCrmService.Execute(retrieve);

		return retrieved.BusinessEntityCollection.BusinessEntities;
	}

    /// <summary>
	/// Retrieve resource from global resource file
    /// </summary>
    public static string GetResource(string key)
    {
        return HttpContext.GetGlobalResourceObject("MobileApplication", key).ToString();
    }

	/// <summary>
	/// Checks to see if the current user has the necessary privilege
	/// </summary>
	/// <param name="entityName"></param>
	/// <param name="privType"></param>
	/// <returns></returns>
    public static bool DoesUserHavePrivilege(string entityName, PrivilegeType privType)
    {
		// Todo: Pass in service?
        MetadataService metadataService = new MetadataService();
        metadataService.Credentials = System.Net.CredentialCache.DefaultCredentials;

		// Retrieve entity so we can get the privilege ID
		EntityMetadata entity = MetadataCache.GetEntityMetadata(entityName);

        Guid entityPrivilegeGuid = Guid.Empty;

        for (int i = 0; i < entity.Privileges.Length; i++)
        {
            if (entity.Privileges[i].PrivilegeType == privType)
            {
                entityPrivilegeGuid = entity.Privileges[i].PrivilegeId;
                break;
            }
        }

		// If the privilege was found, go and see if the user has it
        if (entityPrivilegeGuid != Guid.Empty)
        {
            User currentUser = UserCache.GetCurrentUser();

            return currentUser.GetPrivilege(entityPrivilegeGuid);
        }

        return false;
    }
    
	/// <summary>
	/// Checks to see if the current user has the necessary rights to the specified record
	/// </summary>
	/// <param name="entityName">Schema name of the entity being checked</param>
	/// <param name="recordId">Record ID of the record being checked</param>
	/// <param name="checkedRight">The right being checked</param>
	/// <returns>If user has the appropriate rights</returns>
    public static bool DoesUserHaveRightsToRecord(string entityName, Guid recordId, AccessRights checkedRight)
    {
		// TODO: Pass in service?
        CrmService crmService = new CrmService();
        crmService.Credentials = System.Net.CredentialCache.DefaultCredentials;

		// Set the target being used (record)
        TargetOwnedDynamic target = new TargetOwnedDynamic();
        target.EntityName = entityName;
        target.EntityId = recordId;

		// Set the principal being used (user)
        SecurityPrincipal principal = new SecurityPrincipal();
        principal.Type = SecurityPrincipalType.User;

        User user = UserCache.GetCurrentUser();

        principal.PrincipalId = user.UserId;

		// Retrieve AccessRights
        RetrievePrincipalAccessRequest request = new RetrievePrincipalAccessRequest();
        request.Principal = principal;
        request.Target = target;

        RetrievePrincipalAccessResponse response = (RetrievePrincipalAccessResponse)crmService.Execute(request);

		// Check to see if the user has the right
		return ((response.AccessRights & checkedRight) == checkedRight);
    }

	/// <summary>
	/// Since you can only edit active records, we must check this and handle the edit link accordingly
	/// </summary>
	/// <param name="entityName">Schema name of the entity being checked</param>
	/// <param name="state">State of the current record</param>
	/// <param name="isCustomEntity">Wether the entity is custom</param>
	/// <returns>Wether the current record is in the active state</returns>
    public static bool IsRecordActive(string entityName, string state, bool isCustomEntity)
    {
        // If the entity is active we will allow editting
        // Custom Entities are added to the WSDL every time you build it and do not have a static Enum
        // for State which is why we are using a resource. 
		if ((entityName == EntityName.account.ToString() && (AccountState)Enum.Parse(typeof(AccountState), state, true) == AccountState.Active) ||
            (entityName == EntityName.incident.ToString() && (IncidentState)Enum.Parse(typeof(IncidentState), state, true) == IncidentState.Active) ||
			(entityName == EntityName.contact.ToString() && (ContactState)Enum.Parse(typeof(ContactState), state, true) == ContactState.Active) ||
			(entityName == EntityName.fax.ToString() && (FaxState)Enum.Parse(typeof(FaxState), state, true) == FaxState.Open) ||
			(entityName == EntityName.lead.ToString() && (LeadState)Enum.Parse(typeof(LeadState), state, true) == LeadState.Open) ||
			(entityName == EntityName.letter.ToString() && (LetterState)Enum.Parse(typeof(LetterState), state, true) == LetterState.Open) ||
			(entityName == EntityName.opportunity.ToString() && (OpportunityState)Enum.Parse(typeof(OpportunityState), state, true) == OpportunityState.Open) ||
			(entityName == EntityName.phonecall.ToString() && (PhoneCallState)Enum.Parse(typeof(PhoneCallState), state, true) == PhoneCallState.Open) ||
			(entityName == EntityName.task.ToString() && (TaskState)Enum.Parse(typeof(TaskState), state, true) == TaskState.Open) ||
            (isCustomEntity && state == GetResource("CustomEntityActiveStateText")))            
        {
            return true;
        }        
          
        // Competitor and Customer Address do not have states, so they are always "Active"
        if (entityName == EntityName.competitor.ToString() || entityName == EntityName.customeraddress.ToString())
        {
			return true;
        }

        return false;
    }

	/// <summary>
	/// Some entities do not support editting and this will determine wether to render edit link for the current entity
	/// </summary>
	/// <param name="entityName">Schema name of the entity being checked</param>
	/// <returns>If the entity doesn't support editting</returns>
    public static bool IsEntityReadOnly(string entityName)
    {        
		// If the current entity is one the following, we must disable the edit link
        if (entityName == EntityName.annotation.ToString() ||
            entityName == EntityName.appointment.ToString() ||
            entityName == EntityName.email.ToString() ||
            entityName == EntityName.invoice.ToString() ||
            entityName == EntityName.invoicedetail.ToString() ||
            entityName == EntityName.salesorder.ToString() ||
            entityName == EntityName.salesorderdetail.ToString() ||
            entityName == EntityName.quote.ToString() ||
            entityName == EntityName.quotedetail.ToString() ||
            entityName == EntityName.systemuser.ToString())
        {
            return true;
        }

		// Retrieve entity node
		XmlNode entityNode = MobileUtility.GetEntityNode(entityName);

		if(entityNode != null)
		{
			if(entityNode.Attributes["readOnly"].Value.Equals("1"))
			{
				return true;
			}
		}
		
        return false;
    }

	/// <summary>
	/// Checks to see if the entity exists in the published XML
	/// </summary>
	/// <param name="entityName">Schema name of the entity being checked</param>
	/// <returns>If the entity is configured in the published XML</returns>
    public static bool IsEntityEnabled(string entityName)
    {
		// Retrieve entity node
        XmlNode entityNode = MobileUtility.GetEntityNode(entityName);

		// Check if entity node exists
        if (entityNode != null)
        {
            return true;
        }

        return false;
    }
    
    /// <summary>
    /// Method to get the entity node that the name attribute matches
	/// the value passed in
    /// </summary>
    public static XmlNode GetEntityNode(string entityName)
    {
		// Load the published XML file
		XmlDocument xmlDoc = new XmlDocument();
		xmlDoc.XmlResolver = null;
		xmlDoc.Load(ConfigurationManager.AppSettings["MobileConfigPath"]);

		// Retrieve entity node
		XmlNode entityNode = xmlDoc.DocumentElement.SelectSingleNode(string.Format("Entities/Entity[@name='{0}']", entityName));
    
		return entityNode;
    }
}
