// =======================================================================
//  File: Organization.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Globalization;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

/// <summary>
/// This Organization object holds various information about the user's organization
/// </summary>
public class Organization
{
	private string _currencySymbol = string.Empty;
	private string _dateSeparator = string.Empty;
	private string _numberFormat = string.Empty;

    private int _currencyFormatCode = 0;
	private int _dateFormatCode = 0;
	private int _languageCode = 0;
	private int _negativeFormatCode = 0;

	private CultureInfo _organizationCulture = null;
    private DateTimeFormatInfo _organizationDateTimeFormat = null;
	private NumberFormatInfo _organizationNumberFormat = null;

	/// <summary>
	/// Symbol used as the currency symbol
	/// </summary>
	public string CurrencySymbol
	{
		get
		{
			return _currencySymbol;
		}
	}

	/// <summary>
	/// Symbol used as the date separator
	/// </summary>
	public string DateSeparator
	{
		get
		{
			return _dateSeparator;
		}
	}

	/// <summary>
	/// Two character description of culture used to specify base number formats
	/// </summary>
	public string NumberFormat
	{
		get
		{
			return _numberFormat;
		}
	}

	/// <summary>
	/// Code that determines currency formatting
	/// </summary>
	public int CurrencyFormatCode
	{
		get
		{
			return _currencyFormatCode;
		}
	}

	/// <summary>
	/// Code that determines date formatting 
	/// </summary>
	public int DateFormatCode
	{
		get
		{
			return _dateFormatCode;
		}
	}
	
	/// <summary>
	/// Code for Language of the organization e.g. 1033
	/// </summary>
	public int LanguageCode
	{
		get
		{
			return _languageCode;
		}
	}

	/// <summary>
	/// Code used to handle formatting of negative numbers 
	/// </summary>
	public int NegativeFormatCode
	{
		get
		{
			return _negativeFormatCode;
		}
	}

	/// <summary>
	/// CultureInfo object that contains all necessary formatting 
	/// </summary>
	public CultureInfo OrganizationCulture
	{
		get
		{
			return _organizationCulture;
		}
	}

	/// <summary>
	/// NumberFormatInfo that contains all number formatting
	/// </summary>
	public NumberFormatInfo OrganizationNumberFormat
	{
		get
		{
			return _organizationNumberFormat;
		}
	}   

	/// <summary>
	/// This will set all the base values so we can intialize the culture to use for formatting
	/// </summary>
	/// <param name="userOrganization">Organization retrieved from the OrganizationCache</param>
	public Organization(organization userOrganization)
	{
		// Set base values
		_currencyFormatCode		= userOrganization.currencyformatcode.Value;
		_currencySymbol			= userOrganization.currencysymbol;
		_dateFormatCode			= userOrganization.dateformatcode.Value;
		_dateSeparator			= userOrganization.dateseparator;
		_languageCode			= userOrganization.languagecode.Value;
		_negativeFormatCode		= userOrganization.negativeformatcode.Value;
		_numberFormat			= userOrganization.numberformat;

		// Set up NumberFormatInfo
		InitializeOrganizationNumberFormat();

		// Set all formats on the culture
		InitializeOrganizationCulture();
	}


	/// <summary>
	/// Sets the NumberFormatInfo to be used by the culture
	/// </summary>
	private void InitializeOrganizationNumberFormat()
	{
		//TODO: Should we put some reasoning for all this?
		switch (this.NumberFormat.ToLower(CultureInfo.InvariantCulture))
		{
			case "eu":
				_organizationNumberFormat = CultureInfo.CreateSpecificCulture("de-DE").NumberFormat;
				break;

			case "fr":
				_organizationNumberFormat = CultureInfo.CreateSpecificCulture("fr-FR").NumberFormat;
				_organizationNumberFormat.NumberGroupSeparator = " ";
				break;

			case "nz":
				_organizationNumberFormat = CultureInfo.CreateSpecificCulture("fr-CH").NumberFormat;
				break;

			case "us":
			default:
				_organizationNumberFormat = CultureInfo.CreateSpecificCulture("en-US").NumberFormat;
				break;
		}

		switch (this.CurrencyFormatCode)
		{
			case 0:

				_organizationNumberFormat.CurrencyPositivePattern = 0;

				switch (this.NegativeFormatCode)
				{
					case 0:
						_organizationNumberFormat.CurrencyNegativePattern = 2;
						break;
					case 1:
						_organizationNumberFormat.CurrencyNegativePattern = 2;
						break;
					case 2:
						_organizationNumberFormat.CurrencyNegativePattern = 0;
						break;
					case 3:
						_organizationNumberFormat.CurrencyNegativePattern = 6;
						break;
					case 4:
						_organizationNumberFormat.CurrencyNegativePattern = 6;
						break;
				}

				break;

			case 1:
				_organizationNumberFormat.CurrencyPositivePattern = 2;

				switch (this.NegativeFormatCode)
				{
					case 0:
						_organizationNumberFormat.CurrencyNegativePattern = 12;
						break;
					case 1:
						_organizationNumberFormat.CurrencyNegativePattern = 12;
						break;
					case 2:
						_organizationNumberFormat.CurrencyNegativePattern = 14;
						break;
					case 3:
						_organizationNumberFormat.CurrencyNegativePattern = 13;
						break;
					case 4:
						_organizationNumberFormat.CurrencyNegativePattern = 13;
						break;
				}

				break;

			case 2:
				_organizationNumberFormat.CurrencyPositivePattern = 1;

				switch (this.NegativeFormatCode)
				{
					case 0:
						_organizationNumberFormat.CurrencyNegativePattern = 5;
						break;
					case 1:
						_organizationNumberFormat.CurrencyNegativePattern = 5;
						break;
					case 2:
						_organizationNumberFormat.CurrencyNegativePattern = 4;
						break;
					case 3:
						_organizationNumberFormat.CurrencyNegativePattern = 6;
						break;
					case 4:
						_organizationNumberFormat.CurrencyNegativePattern = 6;
						break;
				}

				break;

			case 3:
				_organizationNumberFormat.CurrencyPositivePattern = 3;

				switch (this.NegativeFormatCode)
				{
					case 0:
						_organizationNumberFormat.CurrencyNegativePattern = 8;
						break;
					case 1:
						_organizationNumberFormat.CurrencyNegativePattern = 8;
						break;
					case 2:
						_organizationNumberFormat.CurrencyNegativePattern = 15;
						break;
					case 3:
						_organizationNumberFormat.CurrencyNegativePattern = 13;
						break;
					case 4:
						_organizationNumberFormat.CurrencyNegativePattern = 13;
						break;
				}

				break;

			default: // Default to US
				_organizationNumberFormat.CurrencyPositivePattern = 0;
				_organizationNumberFormat.CurrencyNegativePattern = 2;

				break;
		}

		switch (this.NegativeFormatCode)
		{
			case 0:
				_organizationNumberFormat.NumberNegativePattern = 1;
				break;

			case 1:
				_organizationNumberFormat.NumberNegativePattern = 2;
				break;

			case 2:
				_organizationNumberFormat.NumberNegativePattern = 0;
				break;

			case 3:
				_organizationNumberFormat.NumberNegativePattern = 3;
				break;

			case 4:
				_organizationNumberFormat.NumberNegativePattern = 4;
				break;

			default:  // Default to US
				_organizationNumberFormat.NumberNegativePattern = 1;
				break;
		}		

		_organizationNumberFormat.CurrencySymbol = this.CurrencySymbol;
	}

	/// <summary>
	/// Intialize CultureInfo object to be used in the application
	/// </summary>
	private void InitializeOrganizationCulture()
	{
		// CRM date formats that are supported
		string[] dateFormats = new string[] { "M/d/yy", "M/d/yyyy", "MM/dd/yy", "MM/dd/yyyy", "d/M/yy", "d/M/yyyy", "dd/MM/yy", "dd/MM/yyyy", "yy/M/d", "yyyy/M/d", "yy/MM/dd", "yyyy/MM/dd" };
		
		_organizationCulture = new CultureInfo(this.LanguageCode);		
		
		_organizationCulture.DateTimeFormat.DateSeparator = this.DateSeparator;
		
		switch (this.DateFormatCode)
		{
			case 1:
			case 3:
			case 5:
			case 7:
			case 9:
			case 11:
				_organizationCulture.DateTimeFormat.ShortDatePattern = dateFormats[this.DateFormatCode].Replace("/", this.OrganizationCulture.DateTimeFormat.DateSeparator);
				break;

			default:  // Default to US
				_organizationCulture.DateTimeFormat.ShortDatePattern = dateFormats[1].Replace("/", this.OrganizationCulture.DateTimeFormat.DateSeparator);
				break;
		}
		
		_organizationCulture.NumberFormat = _organizationNumberFormat;
	}
}
