// =======================================================================
//  File: UserCache.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

/// <summary>
/// This user cache is used to cache CRM User information so it can be easily retrieve from other parts
/// of the application.
/// </summary>
public class UserCache
{
	// Unique Key used to inserts objects into the cache
	private const string CACHE_KEY = "MicrosoftCrmMobileUserKey";

	private UserCache() { }

	static UserCache() { }

	/// <summary>
	/// Gets the current user from cache
	/// </summary>
	/// <returns>Currnet user as user object (not systemuser)</returns>
	public static User GetCurrentUser()
	{
		string userName = null;

		// Retrieve user name
		if (HttpContext.Current != null)
		{
			userName = HttpContext.Current.Request.ServerVariables["AUTH_USER"];

			if (userName.Length == 0)
			{
				userName = null;
			}
		}

		// TODO: This should never happen right?
		if (userName == null)
		{
			userName = System.Security.Principal.WindowsIdentity.GetCurrent().Name;
		}

		return GetCurrentUser(userName);
	}

	/// <summary>
	/// Retrieves current user from cache or by calling the CRM SDK
	/// </summary>
	/// <param name="userName">Name of the user (domain name)</param>
	/// <returns>Currnet user as user object (not systemuser)</returns>
	private static User GetCurrentUser(string userName)
	{
		string userKey = CACHE_KEY + userName;

		// Retrieve user from cache
		User user = HttpContext.Current.Cache[userKey] as User;

		// If the user was found, return it
		if (user != null)
		{
			return user;
		}

		// Create CRM Service with proper credentials
		CrmService crmService = new CrmService();
		crmService.Credentials = System.Net.CredentialCache.DefaultCredentials;

		// Get current user
		WhoAmIRequest whoAmIRequest = new WhoAmIRequest();
		WhoAmIResponse whoAmIResponse = (WhoAmIResponse)crmService.Execute(whoAmIRequest);

		ColumnSet userCols = new ColumnSet();
		userCols.Attributes = new string[] { "systemuserid", "organizationid" };

		// Retrieve systemuser so we can grab the Org Id
		systemuser systemUser = crmService.Retrieve(EntityName.systemuser.ToString(), whoAmIResponse.UserId, userCols) as systemuser;

		// Retrieve user privileges
		RetrieveUserPrivilegesRequest retrievePrivilegeRequest = new RetrieveUserPrivilegesRequest();
		retrievePrivilegeRequest.UserId = whoAmIResponse.UserId;

		RetrieveUserPrivilegesResponse retrievedPrivileges = (RetrieveUserPrivilegesResponse)crmService.Execute(retrievePrivilegeRequest);

		// Set all needed info on the user object
		user = new User();
		user.UserId = systemUser.systemuserid.Value;
		user.OrganizationId = systemUser.organizationid.Value;
		user.UserPrivileges = retrievedPrivileges.RolePrivileges;

		// Insert the user into the cache
		HttpContext.Current.Cache.Insert(userKey, user, null, Cache.NoAbsoluteExpiration, TimeSpan.FromMinutes(15));

		return user;
	}
}
